@@ -46,8 +46,8 @@
 #include <errno.h>
 #include <stdarg.h>
 #include <stdio.h>
-#include <string.h>
 #include <stdlib.h>
+#include <string.h>
 #include <time.h>
 #include <unistd.h>
 
@@ -70,8 +70,8 @@ ROUND_UP_TO_POWER_OF_2(size_t val, size_t pow2) {
 
 static int _autobuf_enlarge(struct autobuf *autobuf, size_t new_size);
 static void _print_hexline(struct autobuf *out, const void *buffer, size_t length);
-static int _vappendf(struct autobuf *autobuf,
-    const char *format, va_list ap, va_list ap2) __attribute__ ((format(printf, 2, 0)));
+static int _vappendf(struct autobuf *autobuf, const char *format, va_list ap, va_list ap2)
+  __attribute__((format(printf, 2, 0)));
 
 /**
  * Initialize an autobuffer and allocate a chunk of memory
@@ -79,8 +79,7 @@ static int _vappendf(struct autobuf *autobuf,
  * @return -1 if an out-of-memory error happened, 0 otherwise
  */
 int
-abuf_init(struct autobuf *autobuf)
-{
+abuf_init(struct autobuf *autobuf) {
   autobuf->_len = 0;
   autobuf->_buf = calloc(1, getpagesize());
   if (autobuf->_buf == NULL) {
@@ -98,8 +97,7 @@ abuf_init(struct autobuf *autobuf)
  * @param autobuf pointer to autobuf object
  */
 void
-abuf_free(struct autobuf *autobuf)
-{
+abuf_free(struct autobuf *autobuf) {
   free(autobuf->_buf);
   memset(autobuf, 0, sizeof(*autobuf));
 }
@@ -114,9 +112,7 @@ abuf_free(struct autobuf *autobuf)
  *   (excluding the \0)
  */
 int
-abuf_vappendf(struct autobuf *autobuf,
-    const char *format, va_list ap)
-{
+abuf_vappendf(struct autobuf *autobuf, const char *format, va_list ap) {
   int result;
   va_list ap2;
 
@@ -141,12 +137,12 @@ abuf_vappendf(struct autobuf *autobuf,
  *   (excluding the \0)
  */
 int
-abuf_appendf(struct autobuf *autobuf, const char *fmt, ...)
-{
+abuf_appendf(struct autobuf *autobuf, const char *fmt, ...) {
   int rc;
   va_list ap;
 
-  if (autobuf == NULL) return 0;
+  if (autobuf == NULL)
+    return 0;
 
   va_start(ap, fmt);
   rc = abuf_vappendf(autobuf, fmt, ap);
@@ -163,13 +159,13 @@ abuf_appendf(struct autobuf *autobuf, const char *fmt, ...)
  *   (excluding the \0)
  */
 int
-abuf_puts(struct autobuf *autobuf, const char *s)
-{
+abuf_puts(struct autobuf *autobuf, const char *s) {
   size_t len;
 
-  if (autobuf == NULL || s == NULL) return 0;
+  if (autobuf == NULL || s == NULL)
+    return 0;
 
-  len  = strlen(s);
+  len = strlen(s);
   if (_autobuf_enlarge(autobuf, autobuf->_len + len + 1) < 0) {
     return -1;
   }
@@ -186,11 +182,11 @@ abuf_puts(struct autobuf *autobuf, const char *s)
  * @return -1 if an out-of-memory error happened, 0 otherwise
  */
 int
-abuf_strftime(struct autobuf *autobuf, const char *format, const struct tm *tm)
-{
+abuf_strftime(struct autobuf *autobuf, const char *format, const struct tm *tm) {
   size_t rc;
 
-  if (autobuf == NULL) return 0;
+  if (autobuf == NULL)
+    return 0;
 
   rc = strftime(autobuf->_buf + autobuf->_len, autobuf->_total - autobuf->_len, format, tm);
   if (rc == 0) {
@@ -222,9 +218,9 @@ abuf_strftime(struct autobuf *autobuf, const char *format, const struct tm *tm)
  * @return -1 if an out-of-memory error happened, 0 otherwise
  */
 int
-abuf_memcpy(struct autobuf *autobuf, const void *p, const size_t len)
-{
-  if (autobuf == NULL || len == 0) return 0;
+abuf_memcpy(struct autobuf *autobuf, const void *p, const size_t len) {
+  if (autobuf == NULL || len == 0)
+    return 0;
 
   if (_autobuf_enlarge(autobuf, autobuf->_len + len) < 0) {
     return -1;
@@ -246,10 +242,9 @@ abuf_memcpy(struct autobuf *autobuf, const void *p, const size_t len)
  * @return -1 if an out-of-memory error happened, 0 otherwise
  */
 int
-abuf_memcpy_prepend(struct autobuf *autobuf,
-    const void *p, const size_t len)
-{
-  if (autobuf == NULL || len == 0) return 0;
+abuf_memcpy_prepend(struct autobuf *autobuf, const void *p, const size_t len) {
+  if (autobuf == NULL || len == 0)
+    return 0;
 
   if (_autobuf_enlarge(autobuf, autobuf->_len + len) < 0) {
     return -1;
@@ -271,11 +266,12 @@ abuf_memcpy_prepend(struct autobuf *autobuf,
  * @param len number of bytes to be removed
  */
 void
-abuf_pull(struct autobuf * autobuf, size_t len) {
+abuf_pull(struct autobuf *autobuf, size_t len) {
   char *p;
   size_t newsize;
 
-  if (autobuf == NULL || len == 0) return;
+  if (autobuf == NULL || len == 0)
+    return;
 
   if (len != autobuf->_len) {
     memmove(autobuf->_buf, &autobuf->_buf[len], autobuf->_len - len);
@@ -345,7 +341,7 @@ _print_hexline(struct autobuf *out, const void *buffer, size_t length) {
       abuf_puts(out, " ");
     }
     if (i < length) {
-      abuf_appendf(out, "%02x", (int) (buf[i]));
+      abuf_appendf(out, "%02x", (int)(buf[i]));
     }
     else {
       abuf_puts(out, "  ");
@@ -361,7 +357,6 @@ _print_hexline(struct autobuf *out, const void *buffer, size_t length) {
       abuf_puts(out, ".");
     }
   }
-
 }
 
 /**
@@ -371,14 +366,13 @@ _print_hexline(struct autobuf *out, const void *buffer, size_t length) {
  * @return -1 if an out-of-memory error happened, 0 otherwise
  */
 static int
-_autobuf_enlarge(struct autobuf *autobuf, size_t new_size)
-{
+_autobuf_enlarge(struct autobuf *autobuf, size_t new_size) {
   char *p;
   size_t roundUpSize;
 
   new_size++;
   if (new_size > autobuf->_total) {
-    roundUpSize = ROUND_UP_TO_POWER_OF_2(new_size+1, getpagesize());
+    roundUpSize = ROUND_UP_TO_POWER_OF_2(new_size + 1, getpagesize());
     p = realloc(autobuf->_buf, roundUpSize);
     if (p == NULL) {
 #ifdef WIN32
@@ -408,12 +402,12 @@ _autobuf_enlarge(struct autobuf *autobuf, size_t new_size)
  *   (excluding the \0)
  */
 static int
-_vappendf(struct autobuf *autobuf,
-    const char *format, va_list ap, va_list ap2) {
+_vappendf(struct autobuf *autobuf, const char *format, va_list ap, va_list ap2) {
   int rc;
   size_t min_size;
 
-  if (autobuf == NULL) return 0;
+  if (autobuf == NULL)
+    return 0;
 
   rc = vsnprintf(autobuf->_buf + autobuf->_len, autobuf->_total - autobuf->_len, format, ap);
   if (rc < 0) {
