@@ -39,6 +39,10 @@
  *
  */
 
+/**
+ * @file
+ */
+
 #ifndef _COMMON_AUTOBUF_H
 #define _COMMON_AUTOBUF_H
 
@@ -68,24 +72,18 @@ struct autobuf {
 
 EXPORT int abuf_init(struct autobuf *autobuf);
 EXPORT void abuf_free(struct autobuf *autobuf);
-EXPORT int abuf_vappendf(struct autobuf *autobuf, const char *fmt,
-    va_list ap) __attribute__ ((format(printf, 2, 0)));
-EXPORT int abuf_appendf(struct autobuf *autobuf, const char *fmt,
-    ...) __attribute__ ((format(printf, 2, 3)));
-EXPORT int abuf_puts(struct autobuf * autobuf, const char *s);
-EXPORT int abuf_strftime(struct autobuf * autobuf,
-    const char *format, const struct tm * tm);
-EXPORT int abuf_memcpy(struct autobuf * autobuf,
-    const void *p, const size_t len);
-EXPORT int abuf_memcpy_prepend(struct autobuf *autobuf,
-    const void *p, const size_t len);
-EXPORT void abuf_pull(struct autobuf * autobuf, size_t len);
-EXPORT void abuf_hexdump(struct autobuf *out,
-    const char *prefix, const void *buffer, size_t length);
+EXPORT int abuf_vappendf(struct autobuf *autobuf, const char *fmt, va_list ap) __attribute__((format(printf, 2, 0)));
+EXPORT int abuf_appendf(struct autobuf *autobuf, const char *fmt, ...) __attribute__((format(printf, 2, 3)));
+EXPORT int abuf_puts(struct autobuf *autobuf, const char *s);
+EXPORT int abuf_strftime(struct autobuf *autobuf, const char *format, const struct tm *tm);
+EXPORT int abuf_memcpy(struct autobuf *autobuf, const void *p, const size_t len);
+EXPORT int abuf_memcpy_prepend(struct autobuf *autobuf, const void *p, const size_t len);
+EXPORT void abuf_pull(struct autobuf *autobuf, size_t len);
+EXPORT void abuf_hexdump(struct autobuf *out, const char *prefix, const void *buffer, size_t length);
 
 /**
  * Clears the content of an autobuf
- * @param autobuf
+ * @param autobuf autobuffer instance
  */
 static INLINE void
 abuf_clear(struct autobuf *autobuf) {
@@ -95,7 +93,7 @@ abuf_clear(struct autobuf *autobuf) {
 }
 
 /**
- * @param autobuf pointer to autobuf
+ * @param autobuf autobuffer instance
  * @return pointer to internal bufffer memory
  */
 static INLINE char *
@@ -104,7 +102,7 @@ abuf_getptr(struct autobuf *autobuf) {
 }
 
 /**
- * @param autobuf pointer to autobuf
+ * @param autobuf autobuffer instance
  * @return number of bytes stored in autobuf
  */
 static INLINE size_t
@@ -113,7 +111,7 @@ abuf_getlen(struct autobuf *autobuf) {
 }
 
 /**
- * @param autobuf pointer to autobuf
+ * @param autobuf autobuffer instance
  * @return number of bytes allocated in buffer
  */
 static INLINE size_t
@@ -122,12 +120,14 @@ abuf_getmax(struct autobuf *autobuf) {
 }
 
 /**
- *
- * @param autobuf
- * @param len
+ * set the length of the autobuffer, will truncate the length
+ * by the total amount of memory in the buffer. Normally used
+ * to shorten a buffer.
+ * @param autobuf autobuffer instance
+ * @param len new length of the autobuffer
  */
 static INLINE void
-abuf_setlen(struct autobuf * autobuf, size_t len) {
+abuf_setlen(struct autobuf *autobuf, size_t len) {
   if (autobuf->_total > len) {
     autobuf->_len = len;
   }
@@ -137,7 +137,7 @@ abuf_setlen(struct autobuf * autobuf, size_t len) {
 
 /**
  * Append a single byte to an autobuffer
- * @param autobuf
+ * @param autobuf autobuffer instance
  * @param c byte to append
  * @return -1 if an error happened, 0 otherwise
  */
@@ -148,7 +148,7 @@ abuf_append_uint8(struct autobuf *autobuf, const uint8_t c) {
 
 /**
  * Append a uint16 to an autobuffer
- * @param autobuf
+ * @param autobuf autobuffer instance
  * @param s uint16 to append
  * @return -1 if an error happened, 0 otherwise
  */
@@ -159,7 +159,7 @@ abuf_append_uint16(struct autobuf *autobuf, const uint16_t s) {
 
 /**
  * Append a uint32 to an autobuffer
- * @param autobuf
+ * @param autobuf autobuffer instance
  * @param l uint32 to append
  * @return -1 if an error happened, 0 otherwise
  */
@@ -169,7 +169,7 @@ abuf_append_uint32(struct autobuf *autobuf, const uint32_t l) {
 }
 
 /**
- * @param autobuf
+ * @param autobuf autobuffer instance
  * @return true if an autobuf function failed
  *  since the last cleanup of the stream
  */
