@@ -50,10 +50,14 @@
 #include "common/avl.h"
 #include "common/avl_comp.h"
 #include "common/common_types.h"
-#include "config/cfg_io.h"
 #include "config/cfg.h"
+#include "config/cfg_io.h"
 
 static int avl_comp_cfgio(const void *txt1, const void *txt2);
+
+/* phy interface conversion function pointer */
+static int (*_get_phy_if)(char *phy_ifname, const char *ifname) = NULL;
+
 /**
  * Initialize a configuration instance
  * @param instance pointer to cfg_instance
@@ -91,7 +95,8 @@ cfg_append_printable_line(struct autobuf *autobuf, const char *fmt, ...) {
   int rv;
   va_list ap;
 
-  if (autobuf == NULL) return 0;
+  if (autobuf == NULL)
+    return 0;
 
   _value = (unsigned char *)abuf_getptr(autobuf) + abuf_getlen(autobuf);
   len = abuf_getlen(autobuf);
@@ -124,8 +129,8 @@ cfg_append_printable_line(struct autobuf *autobuf, const char *fmt, ...) {
 bool
 cfg_is_allowed_key(const char *key) {
   static const char *valid = "_0123456789"
-      "abcdefghijklmnopqrstuvwxyz"
-      "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
+                             "abcdefghijklmnopqrstuvwxyz"
+                             "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
   /* test for [a-zA-Z_][a-zA-Z0-9_]* */
   if (*key >= '0' && *key <= '9') {
     return false;
@@ -156,25 +161,72 @@ cfg_avlcmp_keys(const void *p1, const void *p2) {
   return strcasecmp(str1, str2);
 }
 
+/**
+ * Returns an element of a string array for the CHOICE schema entry
+ * @param idx index of the array
+ * @param ptr pointer to the string array
+ * @return element of the string array at the index
+ */
+const char *
+cfg_get_choice_array_value(size_t idx, const void *ptr) {
+  const char *const *string_array;
+
+  string_array = ptr;
+
+  return string_array[idx];
+}
+
 /**
  * Looks up the index of a string within a string array
  * @param key pointer to string to be looked up in the array
- * @param array pointer to string pointer array
- * @param array_size number of strings in array
+ * @param callback pointer to the callback that returns choice options
+ * @param choice_count number of choices
+ * @param ptr (optional) pointer for choice callback
  * @return index of the string inside the array, -1 if not found
  */
 int
-cfg_get_choice_index(const char *key, const char **array, size_t array_size) {
+cfg_get_choice_index(
+  const char *key, const char *(*callback)(size_t idx, const void *ptr), size_t choices_count, const void *ptr) {
   size_t i;
 
-  for (i=0; i<array_size; i++) {
-    if (strcasecmp(key, array[i]) == 0) {
-      return (int) i;
+  for (i = 0; i < choices_count; i++) {
+    if (strcasecmp(key, callback(i, ptr)) == 0) {
+      return (int)i;
     }
   }
   return -1;
 }
 
+/**
+ * Set a handler to transform a logical interface name into a physical one
+ * @param get_phy_if function pointer to transformer function, NULL to reset
+ */
+void
+cfg_set_ifname_handler(int (*get_phy_if)(char *phy_ifname, const char *ifname)) {
+  _get_phy_if = get_phy_if;
+}
+
+/**
+ * Get a physical interface name from a logical one. If no handler
+ * is available it will fall back to the identity function
+ * (phyiscal = logical name)
+ * @param phy_if target buffer for physical interface name,
+ *    must be IF_NAMESIZE sized
+ * @param ifname logical interface name
+ * @return physical interface name (will never be NULL)
+ */
+const char *
+cfg_get_phy_if(char *phy_if, const char *ifname) {
+  if (_get_phy_if && !_get_phy_if(phy_if, ifname)) {
+    return phy_if;
+  }
+
+  if (phy_if != ifname) {
+    strscpy(phy_if, ifname, IF_NAMESIZE);
+  }
+  return phy_if;
+}
+
 /**
  * AVL tree comparator for case insensitive strings.
  * Custom pointer is the length of the memory to compare.
