@@ -49,10 +49,10 @@
 #include <sys/socket.h>
 
 /* and now the rest of the includes */
-#include <linux/types.h>
-#include <linux/netlink.h>
-#include <linux/genetlink.h>
 #include "nl80211.h"
+#include <linux/genetlink.h>
+#include <linux/netlink.h>
+#include <linux/types.h>
 #include <netlink/attr.h>
 #include <netlink/msg.h>
 #include <sys/uio.h>
@@ -70,9 +70,9 @@
 #include "core/oonf_logging.h"
 #include "core/oonf_subsystem.h"
 #include "subsystems/oonf_class.h"
-#include "subsystems/oonf_interface.h"
 #include "subsystems/oonf_layer2.h"
 #include "subsystems/oonf_timer.h"
+#include "subsystems/os_interface.h"
 #include "subsystems/os_system.h"
 
 #include "nl80211_listener/genl_get_family.h"
@@ -91,6 +91,9 @@
 struct _nl80211_config {
   /*! interval between two series of netlink probes */
   uint64_t interval;
+
+  /*! true if plugin should set multicast rate in the l2 db */
+  bool report_multicast_rate;
 };
 
 /**
@@ -107,70 +110,49 @@ struct _nl80211_query {
    * @param hdr generic message header
    * @param interf netlink interface
    */
-  void (* send)(struct os_system_netlink *nl, struct nlmsghdr *nl_msg,
-      struct genlmsghdr *hdr, struct nl80211_if *interf);
+  void (*send)(
+    struct os_system_netlink *nl, struct nlmsghdr *nl_msg, struct genlmsghdr *hdr, struct nl80211_if *interf);
 
   /**
    * Callback to process incoming netlink data
    * @param interf netlink interface
    * @param hdr netlink message header
    */
-  void (* process)(struct nl80211_if *interf, struct nlmsghdr *hdr);
+  void (*process)(struct nl80211_if *interf, struct nlmsghdr *hdr);
 
   /**
    * Finalize the processing of the netlink query
    * @param interf netlink interface
    */
-  void (* finalize)(struct nl80211_if *interf);
+  void (*finalize)(struct nl80211_if *interf);
 };
 
 /**
  * Index number for nl80211 configuration entries
  */
-enum _nl80211_cfg_idx {
-  IDX_INTERVAL,  //!< IDX_INTERVAL
-  IDX_INTERFACES,//!< IDX_INTERFACES
+enum _nl80211_cfg_idx
+{
+  IDX_INTERVAL,
+  IDX_INTERFACES,
+  IDX_MC_RATE,
 };
 
 /**
  * index numbers for netlink queries done by listener
  */
-enum _if_query {
-  QUERY_START       = 0,//!< QUERY_START
-  QUERY_GET_IF      = 0,//!< QUERY_GET_IF
-  QUERY_GET_WIPHY   = 1,//!< QUERY_GET_WIPHY
-  QUERY_GET_SURVEY  = 2,//!< QUERY_GET_SURVEY
-  QUERY_GET_MPP     = 3,//!< QUERY_GET_MPP
-  QUERY_GET_STATION = 4,//!< QUERY_GET_STATION
-  QUERY_END         = 5,//!< QUERY_END
-
-  QUERY_GET_FAMILY  = 6,//!< QUERY_GET_FAMILY
-
-  QUERY_COUNT,          //!< QUERY_COUNT
-};
-
-static struct _nl80211_query _if_query_ops[QUERY_COUNT] =
+enum _if_query
 {
-    [QUERY_GET_IF] = {
-        NL80211_CMD_NEW_INTERFACE,
-        nl80211_send_get_interface, nl80211_process_get_interface_result, NULL,
-    },
-    [QUERY_GET_WIPHY] = {
-        NL80211_CMD_NEW_WIPHY,
-        nl80211_send_get_wiphy, nl80211_process_get_wiphy_result, nl80211_finalize_get_wiphy,
-    },
-    [QUERY_GET_SURVEY] = {
-        NL80211_CMD_NEW_SURVEY_RESULTS,
-        nl80211_send_get_survey, nl80211_process_get_survey_result, NULL,
-    },
-    [QUERY_GET_MPP] = {
-        NL80211_CMD_NEW_MPATH,
-        nl80211_send_get_mpp, nl80211_process_get_mpp_result, NULL,
-    },
-    [QUERY_GET_STATION] = {
-        NL80211_CMD_NEW_STATION,
-        nl80211_send_get_station_dump, nl80211_process_get_station_dump_result, NULL,
-    },
+  QUERY_START = 0,       //!< QUERY_START
+  QUERY_GET_IF = 0,      //!< QUERY_GET_IF
+  QUERY_GET_WIPHY = 1,   //!< QUERY_GET_WIPHY
+  QUERY_GET_SURVEY = 2,  //!< QUERY_GET_SURVEY
+  QUERY_GET_MPP = 3,     //!< QUERY_GET_MPP
+  QUERY_GET_STATION = 4, //!< QUERY_GET_STATION
+  QUERY_END = 5,         //!< QUERY_END
+
+  QUERY_GET_FAMILY = 6, //!< QUERY_GET_FAMILY
+
+  QUERY_COUNT, //!< QUERY_COUNT
 };
 
 /* prototypes */
@@ -193,22 +175,22 @@ static void _cb_nl_error(uint32_t seq, int error);
 static void _cb_nl_timeout(void);
 static void _cb_nl_done(uint32_t seq);
 
-
 /* configuration */
 static struct cfg_schema_section _if_section = {
-  .type = CFG_INTERFACE_SECTION,
-  .mode = CFG_INTERFACE_SECTION_MODE,
+  CFG_OSIF_SCHEMA_INTERFACE_SECTION_INIT,
+
   .cb_delta_handler = _cb_if_config_changed,
   .entries = NULL,
   .entry_count = 0,
 };
 
 static struct cfg_schema_entry _nl80211_entries[] = {
-  [IDX_INTERVAL] = CFG_MAP_CLOCK_MIN(_nl80211_config, interval, "interval", "1.0",
-      "Interval between two linklayer information updates", 100),
-  [IDX_INTERFACES] = CFG_VALIDATE_PRINTABLE_LEN("if", "",
-      "List of additional interfaces to read nl80211 data from",
-      IF_NAMESIZE, .list=true),
+  [IDX_INTERVAL] = CFG_MAP_CLOCK_MIN(
+    _nl80211_config, interval, "interval", "1.0", "Interval between two linklayer information updates", 100),
+  [IDX_INTERFACES] = CFG_VALIDATE_PRINTABLE_LEN(
+    "if", "", "List of additional interfaces to read nl80211 data from", IF_NAMESIZE, .list = true),
+  [IDX_MC_RATE] = CFG_MAP_BOOL(_nl80211_config, report_multicast_rate, "report_mc_rate", "false",
+    "Activate to write the multicast/broadcast speed into the layer2 database"),
 };
 
 static struct cfg_schema_section _nl80211_section = {
@@ -224,9 +206,9 @@ static struct _nl80211_config _config;
 /* plugin declaration */
 static const char *_dependencies[] = {
   OONF_CLASS_SUBSYSTEM,
-  OONF_INTERFACE_SUBSYSTEM,
   OONF_LAYER2_SUBSYSTEM,
   OONF_TIMER_SUBSYSTEM,
+  OONF_OS_INTERFACE_SUBSYSTEM,
   OONF_OS_SYSTEM_SUBSYSTEM,
 };
 
@@ -258,7 +240,7 @@ static struct os_system_netlink _netlink_handler = {
 };
 
 /* buffer for outgoing netlink message */
-static uint32_t _nl_msgbuffer[UIO_MAXIOV/4];
+static uint32_t _nl_msgbuffer[UIO_MAXIOV / 4];
 static struct nlmsghdr *_nl_msg = (void *)_nl_msgbuffer;
 
 /* netlink nl80211 identification */
@@ -266,7 +248,16 @@ static uint32_t _nl80211_id = 0;
 static uint32_t _nl80211_multicast_group = 0;
 
 /* layer2 metadata */
-static uint32_t _layer2_origin, _layer2_old_origin;
+static struct oonf_layer2_origin _layer2_updated_origin = {
+  .name = "nl80211 updated",
+  .proactive = true,
+  .priority = OONF_LAYER2_ORIGIN_RELIABLE + 1,
+};
+static struct oonf_layer2_origin _layer2_data_origin = {
+  .name = "nl80211",
+  .proactive = true,
+  .priority = OONF_LAYER2_ORIGIN_RELIABLE,
+};
 
 /* current query data */
 static struct nl80211_if *_current_query_if = NULL;
@@ -280,9 +271,7 @@ static struct oonf_timer_class _transmission_timer_info = {
   .periodic = true,
 };
 
-static struct oonf_timer_instance _transmission_timer = {
-  .class = &_transmission_timer_info
-};
+static struct oonf_timer_instance _transmission_timer = { .class = &_transmission_timer_info };
 
 /* nl80211_if handling */
 static struct avl_tree _nl80211_if_tree;
@@ -292,6 +281,44 @@ static struct oonf_class _nl80211_if_class = {
   .size = sizeof(struct nl80211_if),
 };
 
+static const struct _nl80211_query _if_query_ops[QUERY_COUNT] = {
+  [QUERY_GET_IF] =
+    {
+      NL80211_CMD_NEW_INTERFACE,
+      nl80211_send_get_interface,
+      nl80211_process_get_interface_result,
+      NULL,
+    },
+  [QUERY_GET_WIPHY] =
+    {
+      NL80211_CMD_NEW_WIPHY,
+      nl80211_send_get_wiphy,
+      nl80211_process_get_wiphy_result,
+      nl80211_finalize_get_wiphy,
+    },
+  [QUERY_GET_SURVEY] =
+    {
+      NL80211_CMD_NEW_SURVEY_RESULTS,
+      nl80211_send_get_survey,
+      nl80211_process_get_survey_result,
+      NULL,
+    },
+  [QUERY_GET_MPP] =
+    {
+      NL80211_CMD_NEW_MPATH,
+      nl80211_send_get_mpp,
+      nl80211_process_get_mpp_result,
+      NULL,
+    },
+  [QUERY_GET_STATION] =
+    {
+      NL80211_CMD_NEW_STATION,
+      nl80211_send_get_station_dump,
+      nl80211_process_get_station_dump_result,
+      NULL,
+    },
+};
+
 static void
 _early_cfg_init(void) {
   LOG_NL80211 = _nl80211_listener_subsystem.logging;
@@ -312,8 +339,8 @@ _init(void) {
   avl_init(&_nl80211_if_tree, avl_comp_strcasecmp, false);
 
   /* get layer2 origin */
-  _layer2_origin = oonf_layer2_register_origin();
-  _layer2_old_origin = oonf_layer2_register_origin();
+  oonf_layer2_origin_add(&_layer2_updated_origin);
+  oonf_layer2_origin_add(&_layer2_data_origin);
 
   oonf_timer_add(&_transmission_timer_info);
   return 0;
@@ -328,7 +355,8 @@ _cleanup(void) {
   avl_for_each_element_safe(&_nl80211_if_tree, interf, _node, it_if) {
     _nl80211_if_remove(interf);
   }
-  oonf_layer2_cleanup_origin(_layer2_origin);
+  oonf_layer2_origin_remove(&_layer2_updated_origin);
+  oonf_layer2_origin_remove(&_layer2_data_origin);
 
   oonf_timer_stop(&_transmission_timer);
   oonf_timer_remove(&_transmission_timer_info);
@@ -345,9 +373,9 @@ struct oonf_layer2_destination *
 nl80211_add_dst(struct oonf_layer2_neigh *l2neigh, const struct netaddr *dstmac) {
   struct oonf_layer2_destination *dst;
 
-  dst = oonf_layer2_destination_add(l2neigh, dstmac, _layer2_origin);
-  if (dst->origin == _layer2_old_origin) {
-    dst->origin = _layer2_origin;
+  dst = oonf_layer2_destination_add(l2neigh, dstmac, &_layer2_updated_origin);
+  if (dst->origin == &_layer2_data_origin) {
+    dst->origin = &_layer2_updated_origin;
   }
   return dst;
 }
@@ -360,9 +388,8 @@ nl80211_add_dst(struct oonf_layer2_neigh *l2neigh, const struct netaddr *dstmac)
  * @return true if value changed, false otherwise
  */
 bool
-nl80211_change_l2net_data(struct oonf_layer2_net *l2net,
-    enum oonf_layer2_network_index idx, uint64_t value) {
-  return oonf_layer2_change_value(&l2net->data[idx], _layer2_origin, value);
+nl80211_change_l2net_data(struct oonf_layer2_net *l2net, enum oonf_layer2_network_index idx, uint64_t value) {
+  return oonf_layer2_data_set_int64(&l2net->data[idx], &_layer2_updated_origin, value);
 }
 
 /**
@@ -373,9 +400,9 @@ nl80211_change_l2net_data(struct oonf_layer2_net *l2net,
  * @return true if value changed, false otherwise
  */
 bool
-nl80211_change_l2net_neighbor_default(struct oonf_layer2_net *l2net,
-    enum oonf_layer2_neighbor_index idx, uint64_t value) {
-  return oonf_layer2_change_value(&l2net->neighdata[idx], _layer2_origin, value);
+nl80211_change_l2net_neighbor_default(
+  struct oonf_layer2_net *l2net, enum oonf_layer2_neighbor_index idx, uint64_t value) {
+  return oonf_layer2_data_set_int64(&l2net->neighdata[idx], &_layer2_updated_origin, value);
 }
 
 /**
@@ -385,7 +412,7 @@ nl80211_change_l2net_neighbor_default(struct oonf_layer2_net *l2net,
  */
 void
 nl80211_cleanup_l2neigh_data(struct oonf_layer2_neigh *l2neigh) {
-  oonf_layer2_neigh_cleanup(l2neigh, _layer2_old_origin);
+  oonf_layer2_neigh_cleanup(l2neigh, &_layer2_data_origin);
 }
 
 /**
@@ -396,9 +423,16 @@ nl80211_cleanup_l2neigh_data(struct oonf_layer2_neigh *l2neigh) {
  * @return true if value changed, false otherwise
  */
 bool
-nl80211_change_l2neigh_data(struct oonf_layer2_neigh *l2neigh,
-    enum oonf_layer2_neighbor_index idx, uint64_t value) {
-  return oonf_layer2_change_value(&l2neigh->data[idx], _layer2_origin, value);
+nl80211_change_l2neigh_data(struct oonf_layer2_neigh *l2neigh, enum oonf_layer2_neighbor_index idx, uint64_t value) {
+  return oonf_layer2_data_set_int64(&l2neigh->data[idx], &_layer2_updated_origin, value);
+}
+
+/**
+ * @return true if plugin should create a broadcast entry neighbor
+ */
+bool
+nl80211_create_broadcast_neighbor(void) {
+  return _config.report_multicast_rate;
 }
 
 /**
@@ -449,14 +483,15 @@ _nl80211_if_add(const char *name) {
 
   /* initialize interface listener */
   interf->if_listener.name = interf->name;
-  if (oonf_interface_add_listener(&interf->if_listener)) {
-    oonf_layer2_net_remove(interf->l2net, _layer2_origin);
+  if (!os_interface_add(&interf->if_listener)) {
+    oonf_layer2_net_remove(interf->l2net, &_layer2_data_origin);
+    oonf_layer2_net_remove(interf->l2net, &_layer2_updated_origin);
     oonf_class_free(&_nl80211_if_class, interf);
     return NULL;
   }
 
-  /* initialize physical interface */
-  interf->phy_if = -1;
+  /* initialize interface */
+  interf->wifi_phy_if = -1;
 
   OONF_DEBUG(LOG_NL80211, "Add if %s", name);
   avl_insert(&_nl80211_if_tree, &interf->_node);
@@ -470,7 +505,7 @@ _nl80211_if_add(const char *name) {
 static void
 _nl80211_if_remove(struct nl80211_if *interf) {
   avl_remove(&_nl80211_if_tree, &interf->_node);
-  oonf_interface_remove_listener(&interf->if_listener);
+  os_interface_remove(&interf->if_listener);
   oonf_class_free(&_nl80211_if_class, interf);
 }
 
@@ -480,16 +515,19 @@ _nl80211_if_remove(struct nl80211_if *interf) {
 static void
 _cb_if_config_changed(void) {
   struct nl80211_if *interf;
+  const char *ifname;
+  char ifbuf[IF_NAMESIZE];
 
+  ifname = cfg_get_phy_if(ifbuf, _if_section.section_name);
   if (_if_section.pre == NULL) {
-    interf = _nl80211_if_add(_if_section.section_name);
+    interf = _nl80211_if_add(ifname);
     if (interf) {
       interf->_if_section = true;
     }
   }
 
   if (_if_section.post == NULL) {
-    interf = _nl80211_if_get(_if_section.section_name);
+    interf = _nl80211_if_get(ifname);
     if (interf) {
       interf->_if_section = false;
       if (!interf->_nl80211_section) {
@@ -563,16 +601,10 @@ _get_next_query(void) {
 
   /* no query left to do? start again at the first */
   if (!_current_query_if) {
-    int id;
     /* start with first interface and query */
     _current_query_if = avl_first_element(&_nl80211_if_tree, _current_query_if, _node);
     _current_query_number = QUERY_START;
     _current_query_in_progress = true;
-
-    /* switch nl80211 layer2-origin's */
-    id = _layer2_origin;
-    _layer2_origin = _layer2_old_origin;
-    _layer2_old_origin = id;
   }
   else {
     /* next query */
@@ -581,13 +613,18 @@ _get_next_query(void) {
     if (_current_query_number == QUERY_END) {
       /* commit interface data */
       if (_current_query_if->ifdata_changed) {
-        oonf_layer2_net_remove(_current_query_if->l2net, _layer2_old_origin);
+        /* set fixed flags for nl80211 data */
+        oonf_layer2_data_set_bool(
+          &_current_query_if->l2net->data[OONF_LAYER2_NET_MCS_BY_PROBING], &_layer2_updated_origin, true);
+
+        /* cleanup old data and relable new one, then commit everything */
+        oonf_layer2_net_cleanup(_current_query_if->l2net, &_layer2_data_origin, true);
+        oonf_layer2_net_relabel(_current_query_if->l2net, &_layer2_data_origin, &_layer2_updated_origin);
         oonf_layer2_net_commit(_current_query_if->l2net);
         _current_query_if->ifdata_changed = false;
       }
 
-      _current_query_if = avl_next_element_safe(
-          &_nl80211_if_tree, _current_query_if, _node);
+      _current_query_if = avl_next_element_safe(&_nl80211_if_tree, _current_query_if, _node);
       _current_query_number = QUERY_START;
     }
   }
@@ -612,18 +649,20 @@ _trigger_next_netlink_query(void) {
     return;
   }
 
-  /* calculate next interface/query */
-  _get_next_query();
+  /* calculate next interface/query, ignore interfaces that are down */
+  do {
+    _get_next_query();
+  } while (_current_query_if && !_current_query_if->if_listener.data->flags.up);
 
   if (!_current_query_if) {
     /* done with this series of queries, wait for next timer */
-    OONF_DEBUG(LOG_NL80211, "All queries done for all interfaces");
+    OONF_INFO(LOG_NL80211, "All queries done for all interfaces");
     _current_query_in_progress = false;
     return;
   }
 
-  _send_netlink_message(_current_query_if,
-      _current_query_number);
+  OONF_INFO(LOG_NL80211, "Sending query %u to interface %s", _current_query_number, _current_query_if->name);
+  _send_netlink_message(_current_query_if, _current_query_number);
 }
 
 /**
@@ -636,8 +675,7 @@ _cb_nl_message(struct nlmsghdr *hdr) {
 
   gen_hdr = NLMSG_DATA(hdr);
   if (hdr->nlmsg_type == GENL_ID_CTRL && gen_hdr->cmd == CTRL_CMD_NEWFAMILY) {
-    genl_process_get_family_result(hdr,
-        &_nl80211_id, &_nl80211_multicast_group);
+    genl_process_get_family_result(hdr, &_nl80211_id, &_nl80211_multicast_group);
     return;
   }
 
@@ -647,13 +685,11 @@ _cb_nl_message(struct nlmsghdr *hdr) {
   }
 
   if (gen_hdr->cmd != _if_query_ops[_current_query_number].cmd) {
-    OONF_INFO(LOG_NL80211, "Received Nl80211 command %u for query %u (should be %u)",
-        gen_hdr->cmd, _current_query_number,
-        _if_query_ops[_current_query_number].cmd);
+    OONF_INFO(LOG_NL80211, "Received Nl80211 command %u for query %u (should be %u)", gen_hdr->cmd,
+      _current_query_number, _if_query_ops[_current_query_number].cmd);
   }
   else if (_if_query_ops[_current_query_number].process) {
-    OONF_DEBUG(LOG_NL80211, "Received Nl80211 command %u for query %u",
-        gen_hdr->cmd, _current_query_number);
+    OONF_DEBUG(LOG_NL80211, "Received Nl80211 command %u for query %u", gen_hdr->cmd, _current_query_number);
     _if_query_ops[_current_query_number].process(_current_query_if, hdr);
   }
 }
@@ -665,7 +701,7 @@ _cb_nl_message(struct nlmsghdr *hdr) {
  */
 static void
 _cb_nl_error(uint32_t seq __attribute((unused)), int error __attribute((unused))) {
-  OONF_DEBUG(LOG_NL80211, "seq %u: Received error %d", seq, error);
+  OONF_INFO(LOG_NL80211, "seq %u: Received error %d", seq, error);
   if (_nl80211_id && _nl80211_multicast_group) {
     _trigger_next_netlink_query();
   }
@@ -676,7 +712,7 @@ _cb_nl_error(uint32_t seq __attribute((unused)), int error __attribute((unused))
  */
 static void
 _cb_nl_timeout(void) {
-  OONF_DEBUG(LOG_NL80211, "Received timeout");
+  OONF_INFO(LOG_NL80211, "Received timeout");
   if (_nl80211_id && _nl80211_multicast_group) {
     if (_if_query_ops[_current_query_number].finalize) {
       _if_query_ops[_current_query_number].finalize(_current_query_if);
@@ -691,7 +727,7 @@ _cb_nl_timeout(void) {
  */
 static void
 _cb_nl_done(uint32_t seq __attribute((unused))) {
-  OONF_DEBUG(LOG_NL80211, "%u: Received done", seq);
+  OONF_INFO(LOG_NL80211, "%u: Received done", seq);
   if (_nl80211_id && _nl80211_multicast_group) {
     if (_if_query_ops[_current_query_number].finalize) {
       _if_query_ops[_current_query_number].finalize(_current_query_if);
@@ -709,10 +745,8 @@ _cb_config_changed(void) {
   struct nl80211_if *interf;
   const char *str;
 
-  if (cfg_schema_tobin(&_config, _nl80211_section.post,
-      _nl80211_entries, ARRAYSIZE(_nl80211_entries))) {
-    OONF_WARN(LOG_NL80211, "Could not convert "
-        OONF_NL80211_LISTENER_SUBSYSTEM " config to bin");
+  if (cfg_schema_tobin(&_config, _nl80211_section.post, _nl80211_entries, ARRAYSIZE(_nl80211_entries))) {
+    OONF_WARN(LOG_NL80211, "Could not convert " OONF_NL80211_LISTENER_SUBSYSTEM " config to bin");
     return;
   }
 
@@ -720,8 +754,7 @@ _cb_config_changed(void) {
   oonf_timer_set_ext(&_transmission_timer, 1, _config.interval);
 
   /* mark old interfaces for removal */
-  array = cfg_db_get_schema_entry_value(
-      _nl80211_section.pre, &_nl80211_entries[IDX_INTERFACES]);
+  array = cfg_db_get_schema_entry_value(_nl80211_section.pre, &_nl80211_entries[IDX_INTERFACES]);
   if (array && strarray_get_count_c(array) > 0) {
     strarray_for_each_element(array, str) {
       interf = _nl80211_if_get(str);
@@ -733,8 +766,7 @@ _cb_config_changed(void) {
   }
 
   /* create new interfaces and remove mark */
-  array = cfg_db_get_schema_entry_value(
-      _nl80211_section.post, &_nl80211_entries[IDX_INTERFACES]);
+  array = cfg_db_get_schema_entry_value(_nl80211_section.post, &_nl80211_entries[IDX_INTERFACES]);
   if (array && strarray_get_count_c(array) > 0) {
     strarray_for_each_element(array, str) {
       interf = _nl80211_if_add(str);
@@ -746,8 +778,7 @@ _cb_config_changed(void) {
     }
   }
 
-  array = cfg_db_get_schema_entry_value(
-      _nl80211_section.pre, &_nl80211_entries[IDX_INTERFACES]);
+  array = cfg_db_get_schema_entry_value(_nl80211_section.pre, &_nl80211_entries[IDX_INTERFACES]);
   if (array && strarray_get_count_c(array) > 0) {
     strarray_for_each_element(array, str) {
       interf = _nl80211_if_get(str);
