@@ -39,6 +39,10 @@
  *
  */
 
+/**
+ * @file
+ */
+
 #include <stdio.h>
 
 #include "common/avl.h"
@@ -59,20 +63,22 @@
 
 static void _apply_metric(struct nhdp_domain *domain, const char *metric_name);
 static void _remove_metric(struct nhdp_domain *);
-static void _apply_mpr(struct nhdp_domain *domain,
-    const char *mpr_name, uint8_t willingness);
+static void _apply_mpr(struct nhdp_domain *domain, const char *mpr_name, uint8_t willingness);
 static void _remove_mpr(struct nhdp_domain *);
 
-static void _cb_update_everyone_mpr(void);
+static void _cb_update_everyone_routing_mpr(struct nhdp_domain *domain);
+static void _cb_update_everyone_flooding_mpr(struct nhdp_domain *domain);
+
+static bool _recalculate_neighbor_metric(struct nhdp_domain *domain, struct nhdp_neighbor *neigh);
+static bool _recalculate_routing_mpr_set(struct nhdp_domain *domain);
+static bool _recalculate_flooding_mpr_set(void);
 
-static void _recalculate_neighbor_metric(struct nhdp_domain *domain,
-        struct nhdp_neighbor *neigh);
 static const char *_link_to_string(struct nhdp_metric_str *, uint32_t);
 static const char *_path_to_string(struct nhdp_metric_str *, uint32_t, uint8_t);
 static const char *_int_to_string(struct nhdp_metric_str *, struct nhdp_link *);
 
 /* domain class */
-struct oonf_class _domain_class = {
+static struct oonf_class _domain_class = {
   .name = NHDP_CLASS_DOMAIN,
   .size = sizeof(struct nhdp_domain),
 };
@@ -97,15 +103,14 @@ static struct nhdp_domain_metric _no_metric = {
 static struct nhdp_domain_mpr _everyone_mprs = {
   .name = "Everyone MPR",
 
-  .mpr_start = true,
-  .mprs_start = true,
-
-  .update_mpr = _cb_update_everyone_mpr,
+  .update_flooding_mpr = _cb_update_everyone_flooding_mpr,
+  .update_routing_mpr = _cb_update_everyone_routing_mpr,
 };
 
 /* non-default routing domains registered to NHDP */
 static struct list_entity _domain_list;
 static struct list_entity _domain_listener_list;
+static struct list_entity _domain_metric_postprocessor_list;
 
 static size_t _domain_counter = 0;
 
@@ -114,7 +119,7 @@ static struct avl_tree _domain_metrics;
 static struct avl_tree _domain_mprs;
 
 /* flooding domain */
-struct nhdp_domain _flooding_domain;
+static struct nhdp_domain _flooding_domain;
 
 /* NHDP RFC5444 protocol */
 static struct oonf_rfc5444_protocol *_protocol;
@@ -133,6 +138,7 @@ nhdp_domain_init(struct oonf_rfc5444_protocol *p) {
   oonf_class_add(&_domain_class);
   list_init_head(&_domain_list);
   list_init_head(&_domain_listener_list);
+  list_init_head(&_domain_metric_postprocessor_list);
 
   avl_init(&_domain_metrics, avl_comp_strcasecmp, false);
   avl_init(&_domain_mprs, avl_comp_strcasecmp, false);
@@ -152,13 +158,13 @@ void
 nhdp_domain_cleanup(void) {
   struct nhdp_domain *domain, *d_it;
   struct nhdp_domain_listener *listener, *l_it;
+  struct nhdp_domain_metric_postprocessor *processor, *p_it;
   int i;
 
   list_for_each_element_safe(&_domain_list, domain, _node, d_it) {
     /* free allocated TLVs */
-    for (i=0; i<4; i++) {
-      rfc5444_writer_unregister_addrtlvtype(
-          &_protocol->writer, &domain->_metric_addrtlvs[i]);
+    for (i = 0; i < 4; i++) {
+      rfc5444_writer_unregister_addrtlvtype(&_protocol->writer, &domain->_metric_addrtlvs[i]);
     }
 
     /* remove domain */
@@ -166,6 +172,9 @@ nhdp_domain_cleanup(void) {
     oonf_class_free(&_domain_class, domain);
   }
 
+  list_for_each_element_safe(&_domain_metric_postprocessor_list, processor, _node, p_it) {
+    nhdp_domain_metric_postprocessor_remove(processor);
+  }
   list_for_each_element_safe(&_domain_listener_list, listener, _node, l_it) {
     nhdp_domain_listener_remove(listener);
   }
@@ -260,8 +269,7 @@ nhdp_domain_mpr_add(struct nhdp_domain_mpr *mpr) {
     }
   }
   if (_flooding_domain.mpr == &_everyone_mprs) {
-    _apply_mpr(&_flooding_domain,
-        _flooding_domain.mpr_name, _flooding_domain.local_willingness);
+    _apply_mpr(&_flooding_domain, _flooding_domain.mpr_name, _flooding_domain.local_willingness);
   }
   return 0;
 }
@@ -304,6 +312,18 @@ nhdp_domain_listener_remove(struct nhdp_domain_listener *listener) {
   }
 }
 
+void
+nhdp_domain_metric_postprocessor_add(struct nhdp_domain_metric_postprocessor *processor) {
+  list_add_tail(&_domain_metric_postprocessor_list, &processor->_node);
+}
+
+void
+nhdp_domain_metric_postprocessor_remove(struct nhdp_domain_metric_postprocessor *processor) {
+  if (list_is_node_added(&processor->_node)) {
+    list_remove(&processor->_node);
+  }
+}
+
 /**
  * @param ext TLV extension value of MPR/Linkmetrics
  * @return NHDP domain registered to this extension, NULL if not found
@@ -330,16 +350,24 @@ nhdp_domain_init_link(struct nhdp_link *lnk) {
   struct nhdp_link_domaindata *data;
   int i;
 
+  /* initialize flooding MPR settings */
+  lnk->flooding_willingness = RFC7181_WILLINGNESS_NEVER;
+  lnk->local_is_flooding_mpr = false;
+  lnk->neigh_is_flooding_mpr = false;
+
   /* initialize metrics */
-  for (i=0; i<NHDP_MAXIMUM_DOMAINS; i++) {
-    lnk->_domaindata[i].metric.in = RFC7181_METRIC_MAX;
-    lnk->_domaindata[i].metric.out = RFC7181_METRIC_MAX;
+  for (i = 0; i < NHDP_MAXIMUM_DOMAINS; i++) {
+    lnk->_domaindata[i].metric.in = RFC7181_METRIC_INFINITE;
+    lnk->_domaindata[i].metric.out = RFC7181_METRIC_INFINITE;
+    lnk->_domaindata[i].last_metric_change = oonf_clock_getNow();
   }
   list_for_each_element(&_domain_list, domain, _node) {
     data = nhdp_domain_get_linkdata(domain, lnk);
 
-    data->metric.in = domain->metric->incoming_link_start;
-    data->metric.out = domain->metric->outgoing_link_start;
+    if (domain->metric->no_default_handling) {
+      data->metric.in = domain->metric->incoming_link_start;
+      data->metric.out = domain->metric->outgoing_link_start;
+    }
   }
 }
 
@@ -354,16 +382,18 @@ nhdp_domain_init_l2hop(struct nhdp_l2hop *l2hop) {
   int i;
 
   /* initialize metrics */
-  for (i=0; i<NHDP_MAXIMUM_DOMAINS; i++) {
-    l2hop->_domaindata[i].metric.in = RFC7181_METRIC_MAX;
-    l2hop->_domaindata[i].metric.out = RFC7181_METRIC_MAX;
+  for (i = 0; i < NHDP_MAXIMUM_DOMAINS; i++) {
+    l2hop->_domaindata[i].metric.in = RFC7181_METRIC_INFINITE;
+    l2hop->_domaindata[i].metric.out = RFC7181_METRIC_INFINITE;
   }
 
   list_for_each_element(&_domain_list, domain, _node) {
     data = nhdp_domain_get_l2hopdata(domain, l2hop);
 
-    data->metric.in = domain->metric->incoming_2hop_start;
-    data->metric.out = domain->metric->outgoing_2hop_start;
+    if (domain->metric->no_default_handling) {
+      data->metric.in = domain->metric->incoming_2hop_start;
+      data->metric.out = domain->metric->outgoing_2hop_start;
+    }
   }
 }
 
@@ -377,28 +407,26 @@ nhdp_domain_init_neighbor(struct nhdp_neighbor *neigh) {
   struct nhdp_neighbor_domaindata *data;
   int i;
 
-  /* initialize flooding MPR settings */
-  neigh->flooding_willingness = RFC7181_WILLINGNESS_NEVER;
-  neigh->local_is_flooding_mpr = _flooding_domain.mpr->mprs_start;
-  neigh->neigh_is_flooding_mpr = _flooding_domain.mpr->mpr_start;
+  for (i = 0; i < NHDP_MAXIMUM_DOMAINS; i++) {
+    neigh->_domaindata[i].metric.in = RFC7181_METRIC_INFINITE;
+    neigh->_domaindata[i].metric.out = RFC7181_METRIC_INFINITE;
 
-  for (i=0; i<NHDP_MAXIMUM_DOMAINS; i++) {
-    neigh->_domaindata[i].metric.in = RFC7181_METRIC_MAX;
-    neigh->_domaindata[i].metric.out = RFC7181_METRIC_MAX;
+    neigh->_domaindata[i].best_out_link = NULL;
+    neigh->_domaindata[i].best_out_link_metric = RFC7181_METRIC_INFINITE;
+    neigh->_domaindata[i].willingness = RFC7181_WILLINGNESS_NEVER;
+
+    neigh->_domaindata[i].local_is_mpr = false;
+    neigh->_domaindata[i].neigh_is_mpr = false;
   }
 
   /* initialize metrics and mprs */
   list_for_each_element(&_domain_list, domain, _node) {
     data = nhdp_domain_get_neighbordata(domain, neigh);
 
-    data->metric.in = domain->metric->incoming_link_start;
-    data->metric.out = domain->metric->outgoing_link_start;
-
-    data->best_link = NULL;
-
-    data->willingness = RFC7181_WILLINGNESS_NEVER;
-    data->local_is_mpr = domain->mpr->mprs_start;
-    data->neigh_is_mpr = domain->mpr->mpr_start;
+    if (domain->metric->no_default_handling) {
+      data->metric.in = domain->metric->incoming_link_start;
+      data->metric.out = domain->metric->outgoing_link_start;
+    }
   }
 }
 
@@ -410,8 +438,7 @@ nhdp_domain_init_neighbor(struct nhdp_neighbor *neigh) {
  *   must have a length of at least 2
  */
 void
-nhdp_domain_process_metric_linktlv(struct nhdp_domain *domain,
-    struct nhdp_link *lnk, uint8_t *value) {
+nhdp_domain_process_metric_linktlv(struct nhdp_domain *domain, struct nhdp_link *lnk, const uint8_t *value) {
   struct rfc7181_metric_field metric_field;
   uint32_t metric;
 
@@ -433,8 +460,7 @@ nhdp_domain_process_metric_linktlv(struct nhdp_domain *domain,
  * @param value value of metric tlv
  */
 void
-nhdp_domain_process_metric_2hoptlv(struct nhdp_domain *domain,
-    struct nhdp_l2hop *l2hop, uint8_t *value) {
+nhdp_domain_process_metric_2hoptlv(struct nhdp_domain *domain, struct nhdp_l2hop *l2hop, const uint8_t *value) {
   struct rfc7181_metric_field metric_field;
   struct nhdp_l2hop_domaindata *data;
   uint32_t metric;
@@ -452,88 +478,108 @@ nhdp_domain_process_metric_2hoptlv(struct nhdp_domain *domain,
 }
 
 /**
- * Neighborhood changed in terms of metrics or connectivity.
- * This will trigger a MPR set recalculation.
+ * This will trigger a metric recalculation
+ * @param domain NHDP domain of metric change, NULL for all domains
+ * return true if metric changed, false otherwise
  */
-void
-nhdp_domain_neighborhood_changed(void) {
+static bool
+_recalculate_metrics(struct nhdp_domain *domain, struct nhdp_neighbor *neigh, bool trigger) {
   struct nhdp_domain_listener *listener;
-  struct nhdp_domain *domain;
-  struct nhdp_neighbor *neigh;
-  
-  list_for_each_element(&_domain_list, domain, _node) {
-    list_for_each_element(nhdp_db_get_neigh_list(), neigh, _global_node) {
-      _recalculate_neighbor_metric(domain, neigh);
-    }
+  bool changed_metric;
 
-    if (domain->mpr->update_mpr != NULL) {
-      domain->mpr->update_mpr();
-    }
+  changed_metric = false;
+
+  if (trigger) {
+    OONF_DEBUG(LOG_NHDP, "Recalculating metrics set for domain %d", domain ? domain->index : -1);
   }
 
-  // TODO: flooding mpr ?
-  // (Why do we need to consider flooding MPRs here?)
-
-  list_for_each_element(&_domain_listener_list, listener, _node) {
-    if (listener->update) {
-      listener->update(NULL);
+  if (!domain) {
+    list_for_each_element(&_domain_list, domain, _node) {
+      changed_metric |= _recalculate_metrics(domain, neigh, false);
+    }
+    domain = NULL;
+  }
+  else if (!neigh) {
+    list_for_each_element(nhdp_db_get_neigh_list(), neigh, _global_node) {
+      changed_metric |= _recalculate_neighbor_metric(domain, neigh);
     }
   }
+  else {
+    changed_metric |= _recalculate_neighbor_metric(domain, neigh);
+  }
 
-  /* check if we still have routing MPR selectors */
-  OONF_DEBUG(LOG_NHDP, "Checking if we still have routing MPR selectors");
-  _node_is_selected_as_mpr = false;
-
-  list_for_each_element(&_domain_list, domain, _node) {
-    list_for_each_element(nhdp_db_get_neigh_list(), neigh, _global_node) {
-      if (nhdp_domain_get_neighbordata(domain, neigh)->local_is_mpr) {
-        _node_is_selected_as_mpr = true;
-        return;
+  if (trigger && changed_metric) {
+    list_for_each_element(&_domain_listener_list, listener, _node) {
+      /* trigger domain listeners */
+      if (listener->metric_update) {
+        listener->metric_update(domain);
       }
     }
   }
+
+  if (trigger) {
+    OONF_INFO(
+      LOG_NHDP, "Metrics changed for domain %d: %s", domain ? domain->index : -1, changed_metric ? "true" : "false");
+  }
+  return changed_metric;
+}
+
+bool
+nhdp_domain_recalculate_metrics(struct nhdp_domain *domain, struct nhdp_neighbor *neigh) {
+  return _recalculate_metrics(domain, neigh, true);
+}
+
+static void
+_fire_mpr_changed(struct nhdp_domain *domain) {
+  struct nhdp_domain_listener *listener;
+  list_for_each_element(&_domain_listener_list, listener, _node) {
+    /* trigger domain listeners */
+    if (listener->mpr_update) {
+      listener->mpr_update(domain);
+    }
+  }
+}
+
+void
+nhdp_domain_recalculate_mpr(void) {
+  struct nhdp_domain *domain;
+
+  list_for_each_element(&_domain_list, domain, _node) {
+    if (domain->_mpr_outdated) {
+      if (_recalculate_routing_mpr_set(domain)) {
+        domain->mpr->update_routing_mpr(domain);
+        _fire_mpr_changed(domain);
+      }
+      domain->_mpr_outdated = false;
+    }
+  }
+  if (_flooding_domain._mpr_outdated) {
+    if (_recalculate_flooding_mpr_set()) {
+      _flooding_domain.mpr->update_flooding_mpr(&_flooding_domain);
+      _fire_mpr_changed(&_flooding_domain);
+    }
+    _flooding_domain._mpr_outdated = false;
+  }
 }
 
 /**
- * One neighbor changed in terms of metrics or connectivity.
- * This will trigger a MPR set recalculation.
- * @param neigh neighbor where the changed happened
+ * This marks a MPR domain as 'to be recalculated' as soon as a Hello is sent
+ * @param domain NHDP domain
+ * @param neigh neighbor that triggered the recalculation,
+ *   NULL for unspecified neighbor
+ * @return
  */
 void
-nhdp_domain_neighbor_changed(struct nhdp_neighbor *neigh) {
-  struct nhdp_domain_listener *listener;
-  struct nhdp_domain *domain;
-  
-
-  list_for_each_element(&_domain_list, domain, _node) {
-    _recalculate_neighbor_metric(domain, neigh);
-
-    if (domain->mpr->update_mpr != NULL) {
-      domain->mpr->update_mpr();
+nhdp_domain_delayed_mpr_recalculation(struct nhdp_domain *domain, struct nhdp_neighbor *neigh __attribute__((unused))) {
+  if (!domain) {
+    list_for_each_element(&_domain_list, domain, _node) {
+      nhdp_domain_delayed_mpr_recalculation(domain, neigh);
     }
+    nhdp_domain_delayed_mpr_recalculation(&_flooding_domain, neigh);
+    return;
   }
 
-  // TODO: flooding mpr ?
-  // (Why do we need to consider flooding MPRs here?)
-
-  list_for_each_element(&_domain_listener_list, listener, _node) {
-    if (listener->update) {
-      listener->update(neigh);
-    }
-  }
-
-  /* check if we still have routing MPR selectors */
-  OONF_DEBUG(LOG_NHDP, "Checking if we still have routing MPR selectors");
-  _node_is_selected_as_mpr = false;
-
-  list_for_each_element(&_domain_list, domain, _node) {
-    list_for_each_element(nhdp_db_get_neigh_list(), neigh, _global_node) {
-      if (nhdp_domain_get_neighbordata(domain, neigh)->local_is_mpr) {
-        _node_is_selected_as_mpr = true;
-        return;
-      }
-    }
-  }
+  domain->_mpr_outdated = true;
 }
 
 /**
@@ -552,9 +598,7 @@ nhdp_domain_node_is_mpr(void) {
  * @return number of bytes written into destination buffer
  */
 size_t
-nhdp_domain_process_mprtypes_tlv(
-    uint8_t *mprtypes, size_t mprtypes_size,
-    struct rfc5444_reader_tlvblock_entry *tlv) {
+nhdp_domain_process_mprtypes_tlv(uint8_t *mprtypes, size_t mprtypes_size, struct rfc5444_reader_tlvblock_entry *tlv) {
   struct nhdp_domain *domain;
   size_t count;
 
@@ -579,21 +623,22 @@ nhdp_domain_process_mprtypes_tlv(
 
 /**
  * Process an in MPR tlv for a NHDP link
- * @param mprtypes list of extenstions for MPR
+ * @param mprtypes list of extensions for MPR
  * @param mprtypes_size length of mprtypes array
  * @param neigh NHDP neighbor
  * @param tlv MPR tlv context
  */
 void
-nhdp_domain_process_mpr_tlv(uint8_t *mprtypes, size_t mprtypes_size,
-    struct nhdp_neighbor *neigh, struct rfc5444_reader_tlvblock_entry *tlv) {
+nhdp_domain_process_mpr_tlv(
+  uint8_t *mprtypes, size_t mprtypes_size, struct nhdp_link *lnk, struct rfc5444_reader_tlvblock_entry *tlv) {
   struct nhdp_domain *domain;
+  struct nhdp_neighbor *neigh;
   size_t bit_idx, byte_idx;
   size_t i;
 
-  neigh->local_is_flooding_mpr = false;
+  lnk->local_is_flooding_mpr = false;
   list_for_each_element(&_domain_list, domain, _node) {
-    nhdp_domain_get_neighbordata(domain, neigh)->local_is_mpr = false;
+    nhdp_domain_get_neighbordata(domain, lnk->neigh)->local_is_mpr = false;
   }
 
   if (!tlv) {
@@ -601,13 +646,11 @@ nhdp_domain_process_mpr_tlv(uint8_t *mprtypes, size_t mprtypes_size,
   }
 
   /* set flooding MPR flag */
-  neigh->local_is_flooding_mpr =
-      (tlv->single_value[0] & RFC7181_MPR_FLOODING) != 0;
-  OONF_DEBUG(LOG_NHDP_R, "Flooding MPR for neighbor: %s",
-      neigh->local_is_flooding_mpr ? "true" : "false");
+  lnk->local_is_flooding_mpr = (tlv->single_value[0] & RFC7181_MPR_FLOODING) != 0;
+  OONF_DEBUG(LOG_NHDP_R, "Flooding MPR for neighbor: %s", lnk->local_is_flooding_mpr ? "true" : "false");
 
   /* set routing MPR flags */
-  for (i=0; i<mprtypes_size; i++) {
+  for (i = 0; i < mprtypes_size; i++) {
     domain = nhdp_domain_get_by_ext(mprtypes[i]);
     if (domain == NULL) {
       continue;
@@ -619,11 +662,21 @@ nhdp_domain_process_mpr_tlv(uint8_t *mprtypes, size_t mprtypes_size,
       continue;
     }
 
-    nhdp_domain_get_neighbordata(domain, neigh)->local_is_mpr =
-        (tlv->single_value[byte_idx] & (1 << bit_idx)) != 0;
+    nhdp_domain_get_neighbordata(domain, lnk->neigh)->local_is_mpr =
+      (tlv->single_value[byte_idx] & (1 << bit_idx)) != 0;
 
-    OONF_DEBUG(LOG_NHDP_R, "Routing MPR for neighbor in domain %u: %s",
-        domain->ext, nhdp_domain_get_neighbordata(domain, neigh)->local_is_mpr ? "true" : "false");
+    OONF_DEBUG(LOG_NHDP_R, "Routing MPR for neighbor in domain %u: %s", domain->ext,
+      nhdp_domain_get_neighbordata(domain, lnk->neigh)->local_is_mpr ? "true" : "false");
+  }
+
+  _node_is_selected_as_mpr = false;
+  list_for_each_element(&_domain_list, domain, _node) {
+    list_for_each_element(nhdp_db_get_neigh_list(), neigh, _global_node) {
+      if (nhdp_domain_get_neighbordata(domain, neigh)->local_is_mpr) {
+        _node_is_selected_as_mpr = true;
+        return;
+      }
+    }
   }
 }
 
@@ -631,21 +684,20 @@ nhdp_domain_process_mpr_tlv(uint8_t *mprtypes, size_t mprtypes_size,
  * Process an in Willingness tlv and put values into
  * temporary storage in MPR handler object. Call
  * nhdp_domain_store_willingness to permanently store them later.
- * @param mprtypes list of extenstions for MPR
+ * @param mprtypes list of extensions for MPR
  * @param mprtypes_size length of mprtypes array
  * @param tlv Willingness tlv context
  */
 void
 nhdp_domain_process_willingness_tlv(
-    uint8_t *mprtypes, size_t mprtypes_size,
-    struct rfc5444_reader_tlvblock_entry *tlv) {
+  uint8_t *mprtypes, size_t mprtypes_size, struct rfc5444_reader_tlvblock_entry *tlv) {
   struct nhdp_domain *domain;
   size_t idx, i;
   uint8_t value;
 
   _flooding_domain._tmp_willingness = RFC7181_WILLINGNESS_NEVER;
   list_for_each_element(&_domain_list, domain, _node) {
-    domain->_tmp_willingness= RFC7181_WILLINGNESS_NEVER;
+    domain->_tmp_willingness = RFC7181_WILLINGNESS_NEVER;
   }
 
   if (!tlv) {
@@ -653,12 +705,10 @@ nhdp_domain_process_willingness_tlv(
   }
 
   /* copy flooding willingness */
-  _flooding_domain._tmp_willingness
-    = tlv->single_value[0] & RFC7181_WILLINGNESS_MASK;
-  OONF_DEBUG(LOG_NHDP_R, "Received flooding willingness: %u",
-      _flooding_domain._tmp_willingness);
+  _flooding_domain._tmp_willingness = tlv->single_value[0] & RFC7181_WILLINGNESS_MASK;
+  OONF_DEBUG(LOG_NHDP_R, "Received flooding willingness: %u", _flooding_domain._tmp_willingness);
 
-  for (i=0; i<mprtypes_size; i++) {
+  for (i = 0; i < mprtypes_size; i++) {
     domain = nhdp_domain_get_by_ext(mprtypes[i]);
     if (domain == NULL) {
       continue;
@@ -679,8 +729,7 @@ nhdp_domain_process_willingness_tlv(
 
     domain->_tmp_willingness = value;
 
-    OONF_DEBUG(LOG_NHDP_R, "Received routing willingness for domain %u: %u",
-        domain->ext, domain->_tmp_willingness);
+    OONF_DEBUG(LOG_NHDP_R, "Received routing willingness for domain %u: %u", domain->ext, domain->_tmp_willingness);
   }
 }
 
@@ -690,19 +739,17 @@ nhdp_domain_process_willingness_tlv(
  * @param neigh NHDP neighbor
  */
 void
-nhdp_domain_store_willingness(struct nhdp_neighbor *neigh) {
+nhdp_domain_store_willingness(struct nhdp_link *lnk) {
   struct nhdp_neighbor_domaindata *neighdata;
   struct nhdp_domain *domain;
 
-  neigh->flooding_willingness = _flooding_domain._tmp_willingness;
-  OONF_DEBUG(LOG_NHDP_R, "Set flooding willingness: %u",
-      neigh->flooding_willingness);
+  lnk->flooding_willingness = _flooding_domain._tmp_willingness;
+  OONF_DEBUG(LOG_NHDP_R, "Set flooding willingness: %u", lnk->flooding_willingness);
 
   list_for_each_element(&_domain_list, domain, _node) {
-    neighdata = nhdp_domain_get_neighbordata(domain, neigh);
+    neighdata = nhdp_domain_get_neighbordata(domain, lnk->neigh);
     neighdata->willingness = domain->_tmp_willingness;
-    OONF_DEBUG(LOG_NHDP_R, "Set routing willingness for domain %u: %u",
-        domain->ext, neighdata->willingness);
+    OONF_DEBUG(LOG_NHDP_R, "Set routing willingness for domain %u: %u", domain->ext, neighdata->willingness);
   }
 }
 
@@ -713,8 +760,7 @@ nhdp_domain_store_willingness(struct nhdp_neighbor *neigh) {
  * @return number of bytes written into buffer
  */
 size_t
-nhdp_domain_encode_mprtypes_tlvvalue(
-    uint8_t *mprtypes, size_t mprtypes_size) {
+nhdp_domain_encode_mprtypes_tlvvalue(uint8_t *mprtypes, size_t mprtypes_size) {
   struct nhdp_domain *domain;
   size_t count;
 
@@ -739,20 +785,18 @@ nhdp_domain_encode_mprtypes_tlvvalue(
  * @return length of tlvvalue, 0 if an error happened
  */
 size_t
-nhdp_domain_encode_mpr_tlvvalue(
-    uint8_t *tlvvalue, size_t tlvsize, struct nhdp_neighbor *neigh) {
+nhdp_domain_encode_mpr_tlvvalue(uint8_t *tlvvalue, size_t tlvsize, struct nhdp_link *lnk) {
   struct nhdp_domain *domain;
   size_t bit_idx, byte_idx, len;
 
   memset(tlvvalue, 0, tlvsize);
   len = 0;
   /* set flooding MPR flag */
-  if (neigh->neigh_is_flooding_mpr) {
+  if (lnk->neigh_is_flooding_mpr) {
     tlvvalue[0] |= RFC7181_MPR_FLOODING;
   }
 
-  OONF_DEBUG(LOG_NHDP_W, "Set flooding MPR: %s",
-      neigh->neigh_is_flooding_mpr ? "true" : "false");
+  OONF_DEBUG(LOG_NHDP_W, "Set flooding MPR: %s", lnk->neigh_is_flooding_mpr ? "true" : "false");
 
   list_for_each_element(&_domain_list, domain, _node) {
     bit_idx = (domain->index + 1) & 7;
@@ -761,16 +805,16 @@ nhdp_domain_encode_mpr_tlvvalue(
     if (byte_idx >= tlvsize) {
       return 0;
     }
-    if (byte_idx+1 > len) {
+    if (byte_idx + 1 > len) {
       len = byte_idx + 1;
     }
 
-    if (nhdp_domain_get_neighbordata(domain, neigh)->neigh_is_mpr) {
+    if (nhdp_domain_get_neighbordata(domain, lnk->neigh)->neigh_is_mpr) {
       tlvvalue[byte_idx] |= (1 << bit_idx);
     }
 
-    OONF_DEBUG(LOG_NHDP_W, "Set routing MPR for domain %u: %s",
-        domain->ext, nhdp_domain_get_neighbordata(domain, neigh)->neigh_is_mpr ? "true" : "false");
+    OONF_DEBUG(LOG_NHDP_W, "Set routing MPR for domain %u: %s", domain->ext,
+      nhdp_domain_get_neighbordata(domain, lnk->neigh)->neigh_is_mpr ? "true" : "false");
   }
   return len;
 }
@@ -792,8 +836,7 @@ nhdp_domain_encode_willingness_tlvvalue(uint8_t *tlvvalue, size_t tlvsize) {
 
   /* set flooding willingness */
   tlvvalue[0] = _flooding_domain.local_willingness;
-  OONF_DEBUG(LOG_NHDP_W, "Set flooding willingness: %u",
-      _flooding_domain.local_willingness);
+  OONF_DEBUG(LOG_NHDP_W, "Set flooding willingness: %u", _flooding_domain.local_willingness);
 
   /* set routing willingness */
   list_for_each_element(&_domain_list, domain, _node) {
@@ -801,19 +844,21 @@ nhdp_domain_encode_willingness_tlvvalue(uint8_t *tlvvalue, size_t tlvsize) {
     if (idx >= tlvsize) {
       return -1;
     }
-    if (idx+1 > len) {
+    if (idx + 1 > len) {
       len = idx + 1;
     }
 
     value = domain->local_willingness & RFC7181_WILLINGNESS_MASK;
 
-    OONF_DEBUG(LOG_NHDP_W, "Set routing willingness for domain %u: %u"
-        " (%"PRINTF_SIZE_T_SPECIFIER")",
-        domain->ext, value, idx);
-
-    if ((idx & 1) == 0) {
+    if ((domain->index & 1) == 0) {
       value <<= RFC7181_WILLINGNESS_SHIFT;
     }
+
+    OONF_DEBUG(LOG_NHDP_W,
+      "Set routing willingness for domain %u: %x"
+      " (%" PRINTF_SIZE_T_SPECIFIER ")",
+      domain->ext, value, idx);
+
     tlvvalue[idx] |= value;
   }
 
@@ -830,6 +875,13 @@ nhdp_domain_set_flooding_mpr(const char *mpr_name, uint8_t willingness) {
   _apply_mpr(&_flooding_domain, mpr_name, willingness);
 }
 
+/**
+ * @return the virtual flooding domain
+ */
+const struct nhdp_domain *
+nhdp_domain_get_flooding_domain(void) {
+  return &_flooding_domain;
+}
 /**
  * Sets the incoming metric of a link. This is the only function external
  * code should use to commit the calculated metric values to the nhdp db.
@@ -839,82 +891,179 @@ nhdp_domain_set_flooding_mpr(const char *mpr_name, uint8_t willingness) {
  * @return true if metric changed, false otherwise
  */
 bool
-nhdp_domain_set_incoming_metric(struct nhdp_domain_metric *metric,
-    struct nhdp_link *lnk, uint32_t metric_in) {
+nhdp_domain_set_incoming_metric(struct nhdp_domain_metric *metric, struct nhdp_link *lnk, uint32_t metric_in) {
+  struct nhdp_domain_metric_postprocessor *processor;
   struct nhdp_link_domaindata *linkdata;
   struct nhdp_domain *domain;
+  uint32_t new_metric;
   bool changed;
 
   changed = false;
+
   list_for_each_element(&_domain_list, domain, _node) {
     if (domain->metric == metric) {
       linkdata = nhdp_domain_get_linkdata(domain, lnk);
-      changed |= (linkdata->metric.in != metric_in);
-      linkdata->metric.in = metric_in;
+      new_metric = metric_in;
+
+      list_for_each_element(&_domain_metric_postprocessor_list, processor, _node) {
+        new_metric = processor->process_in_metric(domain, lnk, new_metric);
+      }
+
+      if (linkdata->metric.in != new_metric) {
+        changed = true;
+        linkdata->last_metric_change = oonf_clock_getNow();
+      }
+      linkdata->metric.in = new_metric;
     }
   }
   return changed;
 }
 
+/**
+ * @return list of domains
+ */
 struct list_entity *
 nhdp_domain_get_list(void) {
   return &_domain_list;
 }
 
+/**
+ * @return list of event listeners for domain metric/mpr triggers
+ */
 struct list_entity *
 nhdp_domain_get_listener_list(void) {
   return &_domain_listener_list;
 }
 
+static bool
+_recalculate_flooding_mpr_set(void) {
+  struct nhdp_link *lnk;
+
+  list_for_each_element(nhdp_db_get_link_list(), lnk, _global_node) {
+    lnk->_neigh_was_flooding_mpr = lnk->neigh_is_flooding_mpr;
+  }
+
+  _flooding_domain.mpr->update_flooding_mpr(&_flooding_domain);
+
+  list_for_each_element(nhdp_db_get_link_list(), lnk, _global_node) {
+    if (lnk->_neigh_was_flooding_mpr != lnk->neigh_is_flooding_mpr) {
+      OONF_DEBUG(LOG_NHDP, "Flooding domain MPR set changed");
+      return true;
+    }
+  }
+  return false;
+}
+
+/**
+ * Recalculate the MPR set of a NHDP domain
+ * @param domain nhdp domain
+ * @return true if the MPR set changed
+ */
+static bool
+_recalculate_routing_mpr_set(struct nhdp_domain *domain) {
+  struct nhdp_neighbor_domaindata *neighdata;
+  struct nhdp_neighbor *neigh;
+
+  if (!domain->mpr->update_routing_mpr) {
+    return false;
+  }
+
+  /* remember old MPR set */
+  list_for_each_element(nhdp_db_get_neigh_list(), neigh, _global_node) {
+    neighdata = nhdp_domain_get_neighbordata(domain, neigh);
+    neighdata->_neigh_was_mpr = neighdata->neigh_is_mpr;
+  }
+
+  /* update MPR set */
+  domain->mpr->update_routing_mpr(domain);
+
+  /* check for changes */
+  list_for_each_element(nhdp_db_get_neigh_list(), neigh, _global_node) {
+    neighdata = nhdp_domain_get_neighbordata(domain, neigh);
+    if (neighdata->_neigh_was_mpr != neighdata->neigh_is_mpr) {
+      OONF_DEBUG(LOG_NHDP, "Domain ext %u MPR set changed", domain->ext);
+      return true;
+    }
+  }
+  return false;
+}
 
 /**
  * Recalculate the 'best link/metric' values of a neighbor
+ * and check for two-hop outgoing link metric changes
  * @param domain NHDP domain
  * @param neigh NHDP neighbor
+ * @param neighdata NHDP neighbor domaindata
+ * @return true neighbor metric or the two-hop link metrics changed
  */
-static void
-_recalculate_neighbor_metric(
-    struct nhdp_domain *domain,
-    struct nhdp_neighbor *neigh) {
+static bool
+_recalculate_neighbor_metric(struct nhdp_domain *domain, struct nhdp_neighbor *neigh) {
   struct nhdp_link *lnk;
   struct nhdp_link_domaindata *linkdata;
+  struct nhdp_l2hop *l2hop;
+  struct nhdp_l2hop_domaindata *l2hopdata;
   struct nhdp_neighbor_domaindata *neighdata;
-  struct nhdp_metric oldmetric;
+  bool changed;
+#ifdef OONF_LOG_INFO
+  struct netaddr_str nbuf;
+#endif
 
   neighdata = nhdp_domain_get_neighbordata(domain, neigh);
-
-  /* copy old metric value */
-  memcpy(&oldmetric, &neighdata->metric, sizeof(oldmetric));
+  changed = false;
 
   /* reset metric */
   neighdata->metric.in = RFC7181_METRIC_INFINITE;
   neighdata->metric.out = RFC7181_METRIC_INFINITE;
 
   /* reset best link */
-  neighdata->best_link = NULL;
+  neighdata->best_out_link = NULL;
+  neighdata->best_link_ifindex = 0;
+
+  OONF_INFO(LOG_NHDP, "Recalculate neighbor %s metrics (ext %u): old_outgoing=%u",
+    netaddr_to_string(&nbuf, &neigh->originator), domain->ext, neighdata->best_out_link_metric);
 
   /* get best metric */
   list_for_each_element(&neigh->_links, lnk, _neigh_node) {
-    linkdata = nhdp_domain_get_linkdata(domain, lnk);
+    if (lnk->status != NHDP_LINK_SYMMETRIC) {
+      continue;
+    }
 
+    linkdata = nhdp_domain_get_linkdata(domain, lnk);
     if (linkdata->metric.out < neighdata->metric.out) {
+      OONF_DEBUG(LOG_NHDP, "Link on if %s has better outgoing metric: %u", lnk->local_if->os_if_listener.data->name,
+        linkdata->metric.out);
+
       neighdata->metric.out = linkdata->metric.out;
-      neighdata->best_link = lnk;
+      neighdata->best_out_link = lnk;
     }
     if (linkdata->metric.in < neighdata->metric.in) {
+      OONF_DEBUG(LOG_NHDP, "Link on if %s has better incoming metric: %u", lnk->local_if->os_if_listener.data->name,
+        linkdata->metric.in);
       neighdata->metric.in = linkdata->metric.in;
     }
+
+    /* check for changes in outgoing 2-hop metrics */
+    avl_for_each_element(&lnk->_2hop, l2hop, _link_node) {
+      l2hopdata = nhdp_domain_get_l2hopdata(domain, l2hop);
+
+      changed |= l2hopdata->metric.out != l2hopdata->_last_used_outgoing_metric;
+      l2hopdata->_last_used_outgoing_metric = l2hopdata->metric.out;
+    }
   }
 
-  if (neighdata->best_link != NULL) {
-    neighdata->best_link_ifindex =
-        nhdp_interface_get_coreif(neighdata->best_link->local_if)->data.index;
+  if (neighdata->best_out_link != NULL) {
+    linkdata = nhdp_domain_get_linkdata(domain, neighdata->best_out_link);
+
+    OONF_INFO(LOG_NHDP, "Best link: if=%s, link=%s, in=%u, out=%u",
+      nhdp_interface_get_if_listener(neighdata->best_out_link->local_if)->data->name,
+      netaddr_to_string(&nbuf, &neighdata->best_out_link->if_addr), linkdata->metric.in, linkdata->metric.out);
+    neighdata->best_link_ifindex = nhdp_interface_get_if_listener(neighdata->best_out_link->local_if)->data->index;
+
+    changed |= neighdata->best_out_link_metric != linkdata->metric.out;
+    neighdata->best_out_link_metric = linkdata->metric.out;
   }
 
-  if (memcmp(&oldmetric, &neighdata->metric, sizeof(oldmetric)) != 0) {
-    /* mark metric as updated */
-    domain->neighbor_metric_changed = true;
-  }
+  return changed;
 }
 
 /**
@@ -934,8 +1083,7 @@ nhdp_domain_add(uint8_t ext) {
   }
 
   if (_domain_counter == NHDP_MAXIMUM_DOMAINS) {
-    OONF_WARN(LOG_NHDP, "Maximum number of NHDP domains reached: %d",
-        NHDP_MAXIMUM_DOMAINS);
+    OONF_WARN(LOG_NHDP, "Maximum number of NHDP domains reached: %d", NHDP_MAXIMUM_DOMAINS);
     return NULL;
   }
 
@@ -954,18 +1102,17 @@ nhdp_domain_add(uint8_t ext) {
   domain->metric->_refcount++;
 
   /* initialize metric TLVs */
-  for (i=0; i<4; i++) {
+  for (i = 0; i < 4; i++) {
     domain->_metric_addrtlvs[i].type = RFC7181_ADDRTLV_LINK_METRIC;
     domain->_metric_addrtlvs[i].exttype = domain->ext;
 
-    rfc5444_writer_register_addrtlvtype(&_protocol->writer,
-        &domain->_metric_addrtlvs[i], -1);
+    rfc5444_writer_register_addrtlvtype(&_protocol->writer, &domain->_metric_addrtlvs[i], -1);
   }
 
   /* add to domain list */
   list_add_tail(&_domain_list, &domain->_node);
 
-  oonf_class_event(&_domain_class, domain,OONF_OBJECT_ADDED);
+  oonf_class_event(&_domain_class, domain, OONF_OBJECT_ADDED);
   return domain;
 }
 
@@ -985,8 +1132,7 @@ nhdp_domain_add(uint8_t ext) {
  *   maximum number of domains has been reached.
  */
 struct nhdp_domain *
-nhdp_domain_configure(uint8_t ext, const char *metric_name,
-    const char *mpr_name, uint8_t willingness) {
+nhdp_domain_configure(uint8_t ext, const char *metric_name, const char *mpr_name, uint8_t willingness) {
   struct nhdp_domain *domain;
 
   domain = nhdp_domain_add(ext);
@@ -994,12 +1140,10 @@ nhdp_domain_configure(uint8_t ext, const char *metric_name,
     return NULL;
   }
 
-  OONF_DEBUG(LOG_NHDP, "Configure domain %u to metric=%s",
-      domain->index, metric_name);
+  OONF_DEBUG(LOG_NHDP, "Configure domain %u to metric=%s", domain->index, metric_name);
   _apply_metric(domain, metric_name);
 
-  OONF_DEBUG(LOG_NHDP, "Configure domain %u to mpr=%s, willingness=%u",
-      domain->index, mpr_name, willingness);
+  OONF_DEBUG(LOG_NHDP, "Configure domain %u to mpr=%s, willingness=%u", domain->index, mpr_name, willingness);
   _apply_mpr(domain, mpr_name, willingness);
 
   oonf_class_event(&_domain_class, domain, OONF_OBJECT_CHANGED);
@@ -1030,8 +1174,7 @@ _apply_metric(struct nhdp_domain *domain, const char *metric_name) {
   }
 
   /* Handle wildcard metric name first */
-  if (strcasecmp(metric_name, CFG_DOMAIN_ANY_METRIC) == 0
-      && !avl_is_empty(&_domain_metrics)) {
+  if (strcasecmp(metric_name, CFG_DOMAIN_ANY_METRIC_MPR) == 0 && !avl_is_empty(&_domain_metrics)) {
     metric_name = avl_first_element(&_domain_metrics, metric, _node)->name;
   }
 
@@ -1065,7 +1208,7 @@ _remove_metric(struct nhdp_domain *domain) {
   if (!domain->metric->_refcount && domain->metric->disable) {
     domain->metric->disable();
   }
-  strscpy(domain->metric_name, CFG_DOMAIN_NO_METRIC, sizeof(domain->metric_name));
+  strscpy(domain->metric_name, CFG_DOMAIN_NO_METRIC_MPR, sizeof(domain->metric_name));
   domain->metric = &_no_metric;
   domain->metric->_refcount++;
 }
@@ -1096,8 +1239,7 @@ _apply_mpr(struct nhdp_domain *domain, const char *mpr_name, uint8_t willingness
   }
 
   /* Handle wildcard mpr name first */
-  if (strcasecmp(mpr_name, CFG_DOMAIN_ANY_METRIC) == 0
-      && !avl_is_empty(&_domain_mprs)) {
+  if (strcasecmp(mpr_name, CFG_DOMAIN_ANY_METRIC_MPR) == 0 && !avl_is_empty(&_domain_mprs)) {
     mpr_name = avl_first_element(&_domain_mprs, mpr, _node)->name;
   }
 
@@ -1131,30 +1273,30 @@ _remove_mpr(struct nhdp_domain *domain) {
   if (!domain->mpr->_refcount && domain->mpr->disable) {
     domain->mpr->disable();
   }
-  strscpy(domain->mpr_name, CFG_DOMAIN_NO_MPR, sizeof(domain->mpr_name));
+  strscpy(domain->mpr_name, CFG_DOMAIN_NO_METRIC_MPR, sizeof(domain->mpr_name));
   domain->mpr = &_everyone_mprs;
   domain->mpr->_refcount++;
 }
 
 static void
-_cb_update_everyone_mpr(void) {
+_cb_update_everyone_routing_mpr(struct nhdp_domain *domain) {
   struct nhdp_neighbor *neigh;
-  struct nhdp_domain *domain;
   struct nhdp_neighbor_domaindata *domaindata;
 
   list_for_each_element(nhdp_db_get_neigh_list(), neigh, _global_node) {
-    if (_flooding_domain.mpr == &_everyone_mprs) {
-      neigh->neigh_is_flooding_mpr =
-          neigh->flooding_willingness > RFC7181_WILLINGNESS_NEVER;
+    if (domain->mpr == &_everyone_mprs) {
+      domaindata = nhdp_domain_get_neighbordata(domain, neigh);
+      domaindata->neigh_is_mpr = domaindata->willingness > RFC7181_WILLINGNESS_NEVER;
     }
+  }
+}
 
-    list_for_each_element(nhdp_domain_get_list(), domain, _node) {
-      if (domain->mpr == &_everyone_mprs) {
-        domaindata = nhdp_domain_get_neighbordata(domain, neigh);
-        domaindata->neigh_is_mpr =
-            domaindata->willingness > RFC7181_WILLINGNESS_NEVER;
-      }
-    }
+static void
+_cb_update_everyone_flooding_mpr(struct nhdp_domain *domain __attribute__((unused))) {
+  struct nhdp_link *lnk;
+
+  list_for_each_element(nhdp_db_get_link_list(), lnk, _global_node) {
+    lnk->neigh_is_flooding_mpr = lnk->flooding_willingness > RFC7181_WILLINGNESS_NEVER;
   }
 }
 
@@ -1179,16 +1321,14 @@ _link_to_string(struct nhdp_metric_str *buf, uint32_t metric) {
  * @return pointer to string representation of path metric value
  */
 static const char *
-_path_to_string(struct nhdp_metric_str *buf, uint32_t metric,
-    uint8_t hopcount __attribute((unused))) {
+_path_to_string(struct nhdp_metric_str *buf, uint32_t metric, uint8_t hopcount __attribute((unused))) {
   snprintf(buf->buf, sizeof(*buf), "0x%x", metric);
 
   return buf->buf;
 }
 
 static const char *
-_int_to_string(struct nhdp_metric_str *buf,
-    struct nhdp_link *lnk __attribute__((unused))) {
+_int_to_string(struct nhdp_metric_str *buf, struct nhdp_link *lnk __attribute__((unused))) {
   strscpy(buf->buf, "-", sizeof(*buf));
   return buf->buf;
 }
