@@ -75,14 +75,15 @@
 /**
  * Default values for locally attached network parameters
  */
-enum _lan_option_defaults {
-  LAN_DEFAULT_DOMAIN    = 0,//!< LAN_DEFAULT_DOMAIN
-  LAN_DEFAULT_METRIC    = 1,//!< LAN_DEFAULT_METRIC
-  LAN_DEFAULT_DISTANCE  = 2,//!< LAN_DEFAULT_DISTANCE
+enum _lan_option_defaults
+{
+  LAN_DEFAULT_DOMAIN = 0,   //!< LAN_DEFAULT_DOMAIN
+  LAN_DEFAULT_METRIC = 1,   //!< LAN_DEFAULT_METRIC
+  LAN_DEFAULT_DISTANCE = 2, //!< LAN_DEFAULT_DISTANCE
 };
 
 /*! locally attached network option for source-specific prefix */
-#define LAN_OPTION_SRC    "src="
+#define LAN_OPTION_SRC "src="
 
 /*! locally attached network option for outgoing metric */
 #define LAN_OPTION_METRIC "metric="
@@ -91,7 +92,7 @@ enum _lan_option_defaults {
 #define LAN_OPTION_DOMAIN "domain="
 
 /*! locally attached network option for hopcount distance */
-#define LAN_OPTION_DIST   "dist="
+#define LAN_OPTION_DIST "dist="
 
 /**
  * olsrv2 plugin config
@@ -145,9 +146,6 @@ static int _init(void);
 static void _initiate_shutdown(void);
 static void _cleanup(void);
 
-static const char *_parse_lan_parameters(struct os_route_key *prefix,
-		struct _lan_data *dst, const char *src);
-static void _parse_lan_array(struct cfg_named_section *section, bool add);
 static void _cb_generate_tc(struct oonf_timer_instance *);
 
 static void _update_originator(int af_family);
@@ -159,53 +157,38 @@ static void _cb_cfg_domain_changed(void);
 /* subsystem definition */
 static struct cfg_schema_entry _rt_domain_entries[] = {
   CFG_MAP_BOOL(olsrv2_routing_domain, use_srcip_in_routes, "srcip_routes", "true",
-      "Set the source IP of IPv4-routes to a fixed value."),
-  CFG_MAP_INT32_MINMAX(olsrv2_routing_domain, protocol, "protocol", "100",
-      "Protocol number to be used in routing table", 0, false, 1, 254),
-  CFG_MAP_INT32_MINMAX(olsrv2_routing_domain, table, "table", "254",
-      "Routing table number for routes", 0, false, 1, 254),
-  CFG_MAP_INT32_MINMAX(olsrv2_routing_domain, distance, "distance", "2",
-      "Metric Distance to be used in routing table", 0, false, 1, 255),
-  CFG_MAP_BOOL(olsrv2_routing_domain, source_specific, "source_specific", "true",
-      "This domain uses IPv6 source specific routing"),
+    "Set the source IP of IPv4-routes to a fixed value."),
+  CFG_MAP_INT32_MINMAX(
+    olsrv2_routing_domain, protocol, "protocol", "100", "Protocol number to be used in routing table", 0, 1, 254),
+  CFG_MAP_INT32_MINMAX(olsrv2_routing_domain, table, "table", "254", "Routing table number for routes", 0, 1, 254),
+  CFG_MAP_INT32_MINMAX(
+    olsrv2_routing_domain, distance, "distance", "2", "Metric Distance to be used in routing table", 0, 1, 255),
+  CFG_MAP_BOOL(
+    olsrv2_routing_domain, source_specific, "source_specific", "true", "This domain uses IPv6 source specific routing"),
 };
 
 static struct cfg_schema_section _rt_domain_section = {
-  .type = CFG_NHDP_DOMAIN_SECTION,
-  .mode = CFG_SSMODE_NAMED_WITH_DEFAULT,
-  .def_name = CFG_NHDP_DEFAULT_DOMAIN,
+  CFG_NHDP_SCHEMA_DOMAIN_SECTION_INIT,
+
   .cb_delta_handler = _cb_cfg_domain_changed,
   .entries = _rt_domain_entries,
   .entry_count = ARRAYSIZE(_rt_domain_entries),
 };
 
 static struct cfg_schema_entry _olsrv2_entries[] = {
-  CFG_MAP_CLOCK_MIN(_config, tc_interval, "tc_interval", "5.0",
-    "Time between two TC messages", 100),
-  CFG_MAP_CLOCK_MIN(_config, tc_validity, "tc_validity", "300.0",
-    "Validity time of a TC messages", 100),
-  CFG_MAP_CLOCK_MIN(_config, f_hold_time, "forward_hold_time", "300.0",
-    "Holdtime for forwarding set information", 100),
-  CFG_MAP_CLOCK_MIN(_config, p_hold_time, "processing_hold_time", "300.0",
-    "Holdtime for processing set information", 100),
+  CFG_MAP_CLOCK_MIN(_config, tc_interval, "tc_interval", "5.0", "Time between two TC messages", 100),
+  CFG_MAP_CLOCK_MIN(_config, tc_validity, "tc_validity", "300.0", "Validity time of a TC messages", 100),
+  CFG_MAP_CLOCK_MIN(_config, f_hold_time, "forward_hold_time", "300.0", "Holdtime for forwarding set information", 100),
+  CFG_MAP_CLOCK_MIN(
+    _config, p_hold_time, "processing_hold_time", "300.0", "Holdtime for processing set information", 100),
   CFG_MAP_INT64_MINMAX(_config, a_hold_time_factor, "advertisement_hold_time_factor", "3",
-    "Holdtime for TC advertisements as a factor of TC interval time", false, false, 1, 255),
+    "Holdtime for TC advertisements as a factor of TC interval time", false, 1, 255),
   CFG_MAP_BOOL(_config, nhdp_routable, "nhdp_routable", "no",
     "Decides if NHDP interface addresses"
     " are routed to other nodes. 'true' means the 'routable_acl' parameter"
     " will be matched to the addresses to decide."),
-  CFG_MAP_ACL_V46(_config, routable_acl, "routable_acl",
-      OLSRV2_ROUTABLE_IPV4 OLSRV2_ROUTABLE_IPV6 ACL_DEFAULT_ACCEPT,
+  CFG_MAP_ACL_V46(_config, routable_acl, "routable_acl", OLSRV2_ROUTABLE_IPV4 OLSRV2_ROUTABLE_IPV6 ACL_DEFAULT_ACCEPT,
     "Filter to decide which addresses are considered routable"),
-
-  CFG_VALIDATE_LAN(_LOCAL_ATTACHED_NETWORK_KEY, "",
-    "locally attached network, a combination of an"
-    " ip address or prefix followed by an up to four optional parameters"
-    " which define link metric cost, hopcount distance, domain of the prefix"
-    " and the source-prefix ( <"LAN_OPTION_METRIC"...> <"LAN_OPTION_DIST"...>"
-    " <"LAN_OPTION_DOMAIN"<num>/all> <"LAN_OPTION_SRC"...> ).",
-    .list = true),
-
   CFG_MAP_ACL_V46(_config, originator_acl, "originator",
     OLSRV2_ORIGINATOR_IPV4 OLSRV2_ORIGINATOR_IPV6 ACL_DEFAULT_ACCEPT,
     "Filter for router originator addresses (ipv4 and ipv6)"
@@ -267,11 +250,15 @@ static struct oonf_rfc5444_protocol *_protocol;
 
 static bool _generate_tcs = true;
 
+/* TC settings overwrite */
+static uint64_t _overwrite_tc_interval;
+static uint64_t _overwrite_tc_validity;
+
 /* Additional logging sources */
-enum oonf_log_source LOG_OLSRV2;
-enum oonf_log_source LOG_OLSRV2_R;
-enum oonf_log_source LOG_OLSRV2_ROUTING;
-enum oonf_log_source LOG_OLSRV2_W;
+static enum oonf_log_source LOG_OLSRV2;
+static enum oonf_log_source LOG_OLSRV2_R;
+static enum oonf_log_source LOG_OLSRV2_ROUTING;
+static enum oonf_log_source LOG_OLSRV2_W;
 
 /**
  * Initialize additional logging sources for NHDP
@@ -354,6 +341,9 @@ _cleanup(void) {
  */
 uint64_t
 olsrv2_get_tc_interval(void) {
+  if (_overwrite_tc_interval) {
+    return _overwrite_tc_interval;
+  }
   return _olsrv2_config.tc_interval;
 }
 
@@ -362,6 +352,9 @@ olsrv2_get_tc_interval(void) {
  */
 uint64_t
 olsrv2_get_tc_validity(void) {
+  if (_overwrite_tc_validity) {
+    return _overwrite_tc_validity;
+  }
   return _olsrv2_config.tc_validity;
 }
 
@@ -394,8 +387,7 @@ olsrv2_is_routable(struct netaddr *addr) {
  * @return true if TC should be processed, false otherwise
  */
 bool
-olsrv2_mpr_shall_process(
-    struct rfc5444_reader_tlvblock_context *context, uint64_t vtime) {
+olsrv2_mpr_shall_process(struct rfc5444_reader_tlvblock_context *context, uint64_t vtime) {
   enum oonf_duplicate_result dup_result;
   bool process;
 #ifdef OONF_LOG_DEBUG_INFO
@@ -404,24 +396,22 @@ olsrv2_mpr_shall_process(
 
   /* check if message has originator and sequence number */
   if (!context->has_origaddr || !context->has_seqno) {
-    OONF_DEBUG(LOG_OLSRV2, "Do not process message type %u,"
-        " originator or sequence number is missing!",
-        context->msg_type);
+    OONF_DEBUG(LOG_OLSRV2,
+      "Do not process message type %u,"
+      " originator or sequence number is missing!",
+      context->msg_type);
     return false;
   }
 
   /* check forwarding set */
-  dup_result = oonf_duplicate_entry_add(&_protocol->processed_set,
-      context->msg_type, &context->orig_addr,
-      context->seqno, vtime + _olsrv2_config.f_hold_time);
+  dup_result = oonf_duplicate_entry_add(&_protocol->processed_set, context->msg_type, &context->orig_addr,
+    context->seqno, vtime + _olsrv2_config.f_hold_time);
   process = oonf_duplicate_is_new(dup_result);
 
-  OONF_DEBUG(LOG_OLSRV2, "Do %sprocess message type %u from %s"
-      " with seqno %u (dupset result: %u)",
-      process ? "" : "not ",
-      context->msg_type,
-      netaddr_to_string(&buf, &context->orig_addr),
-      context->seqno, dup_result);
+  OONF_DEBUG(LOG_OLSRV2,
+    "Do %sprocess message type %u from %s"
+    " with seqno %u (dupset result: %u)",
+    process ? "" : "not ", context->msg_type, netaddr_to_string(&buf, &context->orig_addr), context->seqno, dup_result);
   return process;
 }
 
@@ -434,8 +424,8 @@ olsrv2_mpr_shall_process(
  * @return true if message was forwarded, false otherwise
  */
 bool
-olsrv2_mpr_shall_forwarding(struct rfc5444_reader_tlvblock_context *context,
-    struct netaddr *source_address, uint64_t vtime) {
+olsrv2_mpr_shall_forwarding(
+  struct rfc5444_reader_tlvblock_context *context, struct netaddr *source_address, uint64_t vtime) {
   struct nhdp_interface *interf;
   struct nhdp_laddr *laddr;
   struct nhdp_neighbor *neigh;
@@ -447,9 +437,10 @@ olsrv2_mpr_shall_forwarding(struct rfc5444_reader_tlvblock_context *context,
 
   /* check if message has originator and sequence number */
   if (!context->has_origaddr || !context->has_seqno) {
-    OONF_DEBUG(LOG_OLSRV2, "Do not forward message type %u,"
-        " originator or sequence number is missing!",
-        context->msg_type);
+    OONF_DEBUG(LOG_OLSRV2,
+      "Do not forward message type %u,"
+      " originator or sequence number is missing!",
+      context->msg_type);
     return false;
   }
 
@@ -459,7 +450,7 @@ olsrv2_mpr_shall_forwarding(struct rfc5444_reader_tlvblock_context *context,
     return false;
   }
 
-  /* checp input source address */
+  /* check input source address */
   if (!source_address) {
     OONF_DEBUG(LOG_OLSRV2, "Do not forward because input source is not set");
     return false;
@@ -471,37 +462,37 @@ olsrv2_mpr_shall_forwarding(struct rfc5444_reader_tlvblock_context *context,
   }
 
   /* check forwarding set */
-  dup_result = oonf_duplicate_entry_add(&_protocol->forwarded_set,
-      context->msg_type, &context->orig_addr,
-      context->seqno, vtime + _olsrv2_config.f_hold_time);
+  dup_result = oonf_duplicate_entry_add(&_protocol->forwarded_set, context->msg_type, &context->orig_addr,
+    context->seqno, vtime + _olsrv2_config.f_hold_time);
   if (!oonf_duplicate_is_new(dup_result)) {
-    OONF_DEBUG(LOG_OLSRV2, "Do not forward message type %u from %s"
-        " with seqno %u (dupset result: %u)",
-        context->msg_type,
-        netaddr_to_string(&buf, &context->orig_addr),
-        context->seqno, dup_result);
+    OONF_DEBUG(LOG_OLSRV2,
+      "Do not forward message type %u from %s"
+      " with seqno %u (dupset result: %u)",
+      context->msg_type, netaddr_to_string(&buf, &context->orig_addr), context->seqno, dup_result);
     return false;
   }
 
   /* get NHDP interface */
   interf = nhdp_interface_get(_protocol->input.interface->name);
   if (interf == NULL) {
-    OONF_DEBUG(LOG_OLSRV2, "Do not forward because NHDP does not handle"
-        " interface '%s'", _protocol->input.interface->name);
+    OONF_DEBUG(LOG_OLSRV2,
+      "Do not forward because NHDP does not handle"
+      " interface '%s'",
+      _protocol->input.interface->name);
     return false;
   }
 
   /* get NHDP link address corresponding to source */
   laddr = nhdp_interface_get_link_addr(interf, source_address);
   if (laddr == NULL) {
-    OONF_DEBUG(LOG_OLSRV2, "Do not forward because source IP %s is"
-        " not a direct neighbor",
-        netaddr_to_string(&buf, source_address));
+    OONF_DEBUG(LOG_OLSRV2,
+      "Do not forward because source IP %s is"
+      " not a direct neighbor",
+      netaddr_to_string(&buf, source_address));
     return false;
   }
 
-  if (netaddr_get_address_family(&context->orig_addr)
-      == netaddr_get_address_family(source_address)) {
+  if (netaddr_get_address_family(&context->orig_addr) == netaddr_get_address_family(source_address)) {
     /* get NHDP neighbor */
     neigh = laddr->link->neigh;
   }
@@ -510,21 +501,20 @@ olsrv2_mpr_shall_forwarding(struct rfc5444_reader_tlvblock_context *context,
     neigh = laddr->link->dualstack_partner->neigh;
   }
   else {
-    OONF_DEBUG(LOG_OLSRV2, "Do not forward because this is a dualstack"
-        " message, but the link source %s is not dualstack capable",
-        netaddr_to_string(&buf, source_address));
+    OONF_DEBUG(LOG_OLSRV2,
+      "Do not forward because this is a dualstack"
+      " message, but the link source %s is not dualstack capable",
+      netaddr_to_string(&buf, source_address));
     return false;
   }
 
   /* forward if this neighbor has selected us as a flooding MPR */
   forward = laddr->link->local_is_flooding_mpr && neigh->symmetric > 0;
-  OONF_DEBUG(LOG_OLSRV2, "Do %sforward message type %u from %s"
-      " with seqno %u (%s/%u)",
-      forward ? "" : "not ",
-      context->msg_type,
-      netaddr_to_string(&buf, &context->orig_addr),
-      context->seqno,
-      laddr->link->local_is_flooding_mpr ? "true" : "false", neigh->symmetric);
+  OONF_DEBUG(LOG_OLSRV2,
+    "Do %sforward message type %u from %s"
+    " with seqno %u (%s/%u)",
+    forward ? "" : "not ", context->msg_type, netaddr_to_string(&buf, &context->orig_addr), context->seqno,
+    laddr->link->local_is_flooding_mpr ? "true" : "false", neigh->symmetric);
   return forward;
 }
 
@@ -535,216 +525,36 @@ olsrv2_mpr_shall_forwarding(struct rfc5444_reader_tlvblock_context *context,
  */
 void
 olsrv2_generate_tcs(bool generate) {
+  uint64_t interval;
+
+  interval = _overwrite_tc_interval;
+  if (!interval) {
+    interval = _olsrv2_config.tc_interval;
+  }
   if (generate && !oonf_timer_is_active(&_tc_timer)) {
-    oonf_timer_set(&_tc_timer, _olsrv2_config.tc_interval);
+    oonf_timer_set(&_tc_timer, interval);
   }
   else if (!generate && oonf_timer_is_active(&_tc_timer)) {
     oonf_timer_stop(&_tc_timer);
   }
 }
 
-/**
- * Schema entry validator for an attached network.
- * See CFG_VALIDATE_ACL_*() macros.
- * @param entry pointer to schema entry
- * @param section_name name of section type and name
- * @param value value of schema entry
- * @param out pointer to autobuffer for validator output
- * @return 0 if validation found no problems, -1 otherwise
- */
-int
-olsrv2_validate_lan(const struct cfg_schema_entry *entry,
-    const char *section_name, const char *value, struct autobuf *out) {
-  struct netaddr_str buf;
-  struct _lan_data data;
-  const char *ptr, *result;
-  struct os_route_key prefix;
+uint64_t
+olsrv2_set_tc_interval(uint64_t interval) {
+  uint64_t old;
 
-  if (value == NULL) {
-    cfg_schema_help_netaddr(entry, out);
-    cfg_append_printable_line(out,
-        "    This value is followed by a list of four optional parameters.");
-    cfg_append_printable_line(out,
-        "    - '"LAN_OPTION_SRC"<prefix>' the source specific prefix of this attached network."
-        " The default is 2.");
-    cfg_append_printable_line(out,
-        "    - '"LAN_OPTION_METRIC"<m>' the link metric of the LAN (between %u and %u)."
-        " The default is 0.", RFC7181_METRIC_MIN, RFC7181_METRIC_MAX);
-    cfg_append_printable_line(out,
-        "    - '"LAN_OPTION_DOMAIN"<d>' the domain of the LAN (between 0 and 255) or 'all'."
-        " The default is all.");
-    cfg_append_printable_line(out,
-        "    - '"LAN_OPTION_DIST"<d>' the hopcount distance of the LAN (between 0 and 255)."
-        " The default is 2.");
-    return 0;
-  }
-
-  ptr = str_cpynextword(buf.buf, value, sizeof(buf));
-  if (cfg_schema_validate_netaddr(entry, section_name, buf.buf, out)) {
-    /* check prefix first */
-    return -1;
-  }
-
-  if (netaddr_from_string(&prefix.dst, buf.buf)) {
-    return -1;
-  }
-
-  result = _parse_lan_parameters(&prefix, &data, ptr);
-  if (result) {
-    cfg_append_printable_line(out, "Value '%s' for entry '%s'"
-        " in section %s has %s",
-        value, entry->key.entry, section_name, result);
-    return -1;
-  }
-
-  if (data.metric < RFC7181_METRIC_MIN || data.metric > RFC7181_METRIC_MAX) {
-    cfg_append_printable_line(out, "Metric %u for prefix %s must be between %u and %u",
-        data.metric, buf.buf, RFC7181_METRIC_MIN, RFC7181_METRIC_MAX);
-    return -1;
-  }
-  if (data.dist > 255) {
-    cfg_append_printable_line(out,
-        "Distance %u for prefix %s must be between 0 and 255", data.dist, buf.buf);
-    return -1;
-  }
-
-  return 0;
+  old = _overwrite_tc_interval;
+  _overwrite_tc_interval = interval;
+  return old;
 }
 
-/**
- * Parse parameters of lan prefix string
- * @param prefix source specific prefix (to store source prefix)
- * @param dst pointer to data structure to store results.
- * @param src source string
- * @return NULL if parser worked without an error, a pointer
- *   to the suffix of the error message otherwise.
- */
-static const char *
-_parse_lan_parameters(struct os_route_key *prefix,
-		struct _lan_data *dst, const char *src) {
-  char buffer[64];
-  const char *ptr, *next;
-  unsigned ext;
+uint64_t
+olsrv2_set_tc_validity(uint64_t interval) {
+  uint64_t old;
 
-  ptr = src;
-  dst->ext = -1;
-  dst->metric = LAN_DEFAULT_METRIC;
-  dst->dist   = LAN_DEFAULT_DISTANCE;
-
-  while (ptr != NULL) {
-    next = str_cpynextword(buffer, ptr, sizeof(buffer));
-
-    if (strncasecmp(buffer, LAN_OPTION_METRIC, 7) == 0) {
-      dst->metric = strtoul(&buffer[7], NULL, 0);
-      if (dst->metric == 0 && errno != 0) {
-        return "an illegal metric parameter";
-      }
-    }
-    else if (strncasecmp(buffer, LAN_OPTION_DOMAIN, 7) == 0) {
-      if (strcasecmp(&buffer[7], "all") == 0) {
-        dst->ext = -1;
-      }
-      else {
-        ext = strtoul(&buffer[7], NULL, 10);
-        if ((ext == 0 && errno != 0) || ext > 255) {
-          return "an illegal domain parameter";
-        }
-        dst->ext = ext;
-      }
-    }
-    else if (strncasecmp(buffer, LAN_OPTION_DIST, 5) == 0) {
-      dst->dist = strtoul(&buffer[5], NULL, 10);
-      if (dst->dist == 0 && errno != 0) {
-        return "an illegal distance parameter";
-      }
-    }
-    else if (strncasecmp(buffer, LAN_OPTION_SRC, 4) == 0) {
-      if (netaddr_from_string(&prefix->src, &buffer[4])) {
-        return "an illegal source prefix";
-      }
-      if (netaddr_get_address_family(&prefix->dst)
-            != netaddr_get_address_family(&prefix->src)) {
-    	  return "an illegal source prefix address type";
-      }
-      if (!os_routing_supports_source_specific(
-          netaddr_get_address_family(&prefix->dst))) {
-        return "an unsupported sourc specific prefix";
-      }
-    }
-    else {
-      return "an unknown parameter";
-    }
-    ptr = next;
-  }
-  return NULL;
-}
-
-/**
- * Takes a named configuration section, extracts the attached network
- * array and apply it
- * @param section pointer to configuration section.
- * @param add true if new lan entries should be created, false if
- *   existing entries should be removed.
- */
-static void
-_parse_lan_array(struct cfg_named_section *section, bool add) {
-  struct netaddr_str addr_buf;
-  struct netaddr addr;
-  struct os_route_key prefix;
-  struct _lan_data data;
-  struct nhdp_domain *domain;
-
-  const char *value, *ptr;
-  struct cfg_entry *entry;
-
-  if (section == NULL) {
-    return;
-  }
-
-  entry = cfg_db_get_entry(section, _LOCAL_ATTACHED_NETWORK_KEY);
-  if (entry == NULL) {
-    return;
-  }
-
-  strarray_for_each_element(&entry->val, value) {
-    /* extract data */
-    ptr = str_cpynextword(addr_buf.buf, value, sizeof(addr_buf));
-    if (netaddr_from_string(&addr, addr_buf.buf)) {
-      continue;
-    }
-
-    os_routing_init_sourcespec_prefix(&prefix, &addr);
-
-    /* truncate address */
-    netaddr_truncate(&prefix.dst, &prefix.dst);
-
-    if (_parse_lan_parameters(&prefix, &data, ptr)) {
-      continue;
-    }
-
-    if (data.ext == -1) {
-      list_for_each_element(nhdp_domain_get_list(), domain, _node) {
-        if (add) {
-          olsrv2_lan_add(domain, &prefix, data.metric, data.dist);
-        }
-        else {
-          olsrv2_lan_remove(domain, &prefix);
-        }
-      }
-    }
-    else {
-      domain = nhdp_domain_add(data.ext);
-      if (!domain) {
-        continue;
-      }
-      if (add) {
-        olsrv2_lan_add(domain, &prefix, data.metric, data.dist);
-      }
-      else {
-        olsrv2_lan_remove(domain, &prefix);
-      }
-    }
-  }
+  old = _overwrite_tc_validity;
+  _overwrite_tc_validity = interval;
+  return old;
 }
 
 /**
@@ -773,43 +583,36 @@ _get_addr_priority(const struct netaddr *addr) {
 
   if (!netaddr_acl_check_accept(&_olsrv2_config.originator_acl, addr)) {
     /* does not match the acl */
-    OONF_DEBUG(LOG_OLSRV2, "check priority for %s: 0 (not in ACL)",
-        netaddr_to_string(&nbuf, addr));
+    OONF_DEBUG(LOG_OLSRV2, "check priority for %s: 0 (not in ACL)", netaddr_to_string(&nbuf, addr));
     return 0;
   }
 
-
   if (netaddr_get_address_family(addr) == AF_INET) {
     if (netaddr_is_in_subnet(&NETADDR_IPV4_LINKLOCAL, addr)) {
       /* linklocal */
-      OONF_DEBUG(LOG_OLSRV2, "check priority for %s: 1 (linklocal)",
-          netaddr_to_string(&nbuf, addr));
+      OONF_DEBUG(LOG_OLSRV2, "check priority for %s: 1 (linklocal)", netaddr_to_string(&nbuf, addr));
       return 1;
     }
 
     /* routable */
-    OONF_DEBUG(LOG_OLSRV2, "check priority for %s: 2 (routable)",
-        netaddr_to_string(&nbuf, addr));
+    OONF_DEBUG(LOG_OLSRV2, "check priority for %s: 2 (routable)", netaddr_to_string(&nbuf, addr));
     return 2;
   }
 
   if (netaddr_get_address_family(addr) == AF_INET6) {
     if (netaddr_is_in_subnet(&NETADDR_IPV6_LINKLOCAL, addr)) {
       /* linklocal */
-      OONF_DEBUG(LOG_OLSRV2, "check priority for %s: 1 (linklocal)",
-          netaddr_to_string(&nbuf, addr));
+      OONF_DEBUG(LOG_OLSRV2, "check priority for %s: 1 (linklocal)", netaddr_to_string(&nbuf, addr));
       return 1;
     }
 
     /* routable */
-    OONF_DEBUG(LOG_OLSRV2, "check priority for %s: 2 (routable)",
-        netaddr_to_string(&nbuf, addr));
+    OONF_DEBUG(LOG_OLSRV2, "check priority for %s: 2 (routable)", netaddr_to_string(&nbuf, addr));
     return 2;
   }
 
   /* unknown */
-  OONF_DEBUG(LOG_OLSRV2, "check priority for %s: 0 (unknown)",
-      netaddr_to_string(&nbuf, addr));
+  OONF_DEBUG(LOG_OLSRV2, "check priority for %s: 0 (unknown)", netaddr_to_string(&nbuf, addr));
 
   return 0;
 }
@@ -832,8 +635,7 @@ _update_originator(int af_family) {
   struct netaddr_str buf;
 #endif
 
-  OONF_DEBUG(LOG_OLSRV2, "Updating OLSRV2 %s originator",
-      af_family == AF_INET ? "ipv4" : "ipv6");
+  OONF_DEBUG(LOG_OLSRV2, "Updating OLSRV2 %s originator", af_family == AF_INET ? "ipv4" : "ipv6");
 
   originator = olsrv2_originator_get(af_family);
 
@@ -870,8 +672,7 @@ _update_originator(int af_family) {
   }
 
   if (new_priority > old_priority) {
-    OONF_DEBUG(LOG_OLSRV2, "Set originator to %s",
-        netaddr_to_string(&buf, &new_originator));
+    OONF_DEBUG(LOG_OLSRV2, "Set originator to %s", netaddr_to_string(&buf, &new_originator));
     olsrv2_originator_set(&new_originator);
   }
 }
@@ -893,26 +694,19 @@ _cb_if_event(struct os_interface_listener *if_listener __attribute__((unused)))
  */
 static void
 _cb_cfg_olsrv2_changed(void) {
-  if (cfg_schema_tobin(&_olsrv2_config, _olsrv2_section.post,
-      _olsrv2_entries, ARRAYSIZE(_olsrv2_entries))) {
+  if (cfg_schema_tobin(&_olsrv2_config, _olsrv2_section.post, _olsrv2_entries, ARRAYSIZE(_olsrv2_entries))) {
     OONF_WARN(LOG_OLSRV2, "Cannot convert OLSRV2 configuration.");
     return;
   }
 
   /* set tc timer interval */
-  if (_generate_tcs) {
+  if (_generate_tcs && _overwrite_tc_interval == 0) {
     oonf_timer_set(&_tc_timer, _olsrv2_config.tc_interval);
   }
 
   /* check if we have to change the originators */
   _update_originator(AF_INET);
   _update_originator(AF_INET6);
-
-  /* run through all pre-update LAN entries and remove them */
-  _parse_lan_array(_olsrv2_section.pre, false);
-
-  /* run through all post-update LAN entries and add them */
-  _parse_lan_array(_olsrv2_section.post, true);
 }
 
 /**
@@ -942,8 +736,7 @@ _cb_cfg_domain_changed(void) {
   }
 
   memset(&rtdomain, 0, sizeof(rtdomain));
-  if (cfg_schema_tobin(&rtdomain, _rt_domain_section.post,
-      _rt_domain_entries, ARRAYSIZE(_rt_domain_entries))) {
+  if (cfg_schema_tobin(&rtdomain, _rt_domain_section.post, _rt_domain_entries, ARRAYSIZE(_rt_domain_entries))) {
     OONF_WARN(LOG_OLSRV2, "Cannot convert OLSRV2 routing domain parameters.");
     return;
   }
