@@ -39,20 +39,21 @@
  *
  */
 
+/**
+ * @file
+ */
+
 #ifndef OLSRV2_WRITER_H_
 #define OLSRV2_WRITER_H_
 
 #include "common/common_types.h"
 #include "subsystems/oonf_rfc5444.h"
 
-int olsrv2_writer_init(struct oonf_rfc5444_protocol *)
-  __attribute__((warn_unused_result));
+int olsrv2_writer_init(struct oonf_rfc5444_protocol *) __attribute__((warn_unused_result));
 void olsrv2_writer_cleanup(void);
 
 EXPORT void olsrv2_writer_send_tc(void);
 EXPORT void olsrv2_writer_set_forwarding_selector(
-    bool (*forward_target_selector)(struct rfc5444_writer_target *,
-      struct rfc5444_reader_tlvblock_context *context,
-      const uint8_t *buffer, size_t len));
+  bool (*forward_target_selector)(struct rfc5444_writer_target *, struct rfc5444_reader_tlvblock_context *context));
 
 #endif /* OLSRV2_WRITER_H_ */
