@@ -51,14 +51,14 @@
 #include <linux/rtnetlink.h>
 #include <sys/uio.h>
 
-#include "common/common_types.h"
 #include "common/avl.h"
 #include "common/avl_comp.h"
+#include "common/common_types.h"
 #include "core/oonf_subsystem.h"
 #include "subsystems/os_system.h"
 
-#include "subsystems/os_routing.h"
 #include "subsystems/os_linux/os_routing_linux.h"
+#include "subsystems/os_routing.h"
 
 /* Definitions */
 #define LOG_OS_ROUTING _oonf_os_routing_subsystem.logging
@@ -78,12 +78,10 @@ struct route_type_translation {
 static int _init(void);
 static void _cleanup(void);
 
-static int _routing_set(struct nlmsghdr *msg, struct os_route *route,
-    unsigned char rt_scope);
+static int _routing_set(struct nlmsghdr *msg, struct os_route *route, unsigned char rt_scope);
 
 static void _routing_finished(struct os_route *route, int error);
 static void _cb_rtnetlink_message(struct nlmsghdr *);
-static void _cb_rtnetlink_event_message(struct nlmsghdr *);
 static void _cb_rtnetlink_error(uint32_t seq, int err);
 static void _cb_rtnetlink_done(uint32_t seq);
 static void _cb_rtnetlink_timeout(void);
@@ -103,22 +101,13 @@ static struct oonf_subsystem _oonf_os_routing_subsystem = {
 DECLARE_OONF_PLUGIN(_oonf_os_routing_subsystem);
 
 /* translation table between route types */
-static struct route_type_translation _type_translation[] = {
-    { OS_ROUTE_UNICAST, RTN_UNICAST },
-    { OS_ROUTE_LOCAL, RTN_LOCAL },
-    { OS_ROUTE_BROADCAST, RTN_BROADCAST },
-    { OS_ROUTE_MULTICAST, RTN_MULTICAST },
-    { OS_ROUTE_THROW, RTN_THROW },
-    { OS_ROUTE_UNREACHABLE, RTN_UNREACHABLE },
-    { OS_ROUTE_PROHIBIT, RTN_PROHIBIT },
-    { OS_ROUTE_BLACKHOLE, RTN_BLACKHOLE },
-    { OS_ROUTE_NAT, RTN_NAT }
-};
+static struct route_type_translation _type_translation[] = { { OS_ROUTE_UNICAST, RTN_UNICAST },
+  { OS_ROUTE_LOCAL, RTN_LOCAL }, { OS_ROUTE_BROADCAST, RTN_BROADCAST }, { OS_ROUTE_MULTICAST, RTN_MULTICAST },
+  { OS_ROUTE_THROW, RTN_THROW }, { OS_ROUTE_UNREACHABLE, RTN_UNREACHABLE }, { OS_ROUTE_PROHIBIT, RTN_PROHIBIT },
+  { OS_ROUTE_BLACKHOLE, RTN_BLACKHOLE }, { OS_ROUTE_NAT, RTN_NAT } };
 
 /* netlink socket for route set/get commands */
-static const uint32_t _rtnetlink_mcast[] = {
-  RTNLGRP_IPV4_ROUTE, RTNLGRP_IPV6_ROUTE
-};
+static const uint32_t _rtnetlink_mcast[] = { RTNLGRP_IPV4_ROUTE, RTNLGRP_IPV6_ROUTE };
 
 static struct os_system_netlink _rtnetlink_socket = {
   .name = "routing",
@@ -129,30 +118,23 @@ static struct os_system_netlink _rtnetlink_socket = {
   .cb_timeout = _cb_rtnetlink_timeout,
 };
 
-static struct os_system_netlink _rtnetlink_event_socket = {
-  .name = "routing listener",
-  .used_by = &_oonf_os_routing_subsystem,
-  .cb_message = _cb_rtnetlink_event_message,
-};
-
 static struct avl_tree _rtnetlink_feedback;
 static struct list_entity _rtnetlink_listener;
 
 /* default wildcard route */
-static const struct os_route_parameter OS_ROUTE_WILDCARD = {
-  .family = AF_UNSPEC,
+static const struct os_route_parameter OS_ROUTE_WILDCARD = { .family = AF_UNSPEC,
   .src_ip = { ._type = AF_UNSPEC },
   .gw = { ._type = AF_UNSPEC },
   .type = OS_ROUTE_UNDEFINED,
-  .key = {
+  .key =
+    {
       .dst = { ._type = AF_UNSPEC },
       .src = { ._type = AF_UNSPEC },
-  },
+    },
   .table = RT_TABLE_UNSPEC,
   .metric = -1,
   .protocol = RTPROT_UNSPEC,
-  .if_index = 0
-};
+  .if_index = 0 };
 
 /* kernel version check */
 static bool _is_kernel_3_11_0_or_better;
@@ -166,21 +148,14 @@ _init(void) {
   if (os_system_linux_netlink_add(&_rtnetlink_socket, NETLINK_ROUTE)) {
     return -1;
   }
-
-  if (os_system_linux_netlink_add(&_rtnetlink_event_socket, NETLINK_ROUTE)) {
+  if (os_system_linux_netlink_add_mc(&_rtnetlink_socket, _rtnetlink_mcast, ARRAYSIZE(_rtnetlink_mcast))) {
     os_system_linux_netlink_remove(&_rtnetlink_socket);
     return -1;
   }
-
-  if (os_system_linux_netlink_add_mc(&_rtnetlink_event_socket, _rtnetlink_mcast, ARRAYSIZE(_rtnetlink_mcast))) {
-    os_system_linux_netlink_remove(&_rtnetlink_socket);
-    os_system_linux_netlink_remove(&_rtnetlink_event_socket);
-    return -1;
-  }
   avl_init(&_rtnetlink_feedback, avl_comp_uint32, false);
   list_init_head(&_rtnetlink_listener);
 
-  _is_kernel_3_11_0_or_better = os_system_linux_is_minimal_kernel(3,11,0);
+  _is_kernel_3_11_0_or_better = os_system_linux_is_minimal_kernel(3, 11, 0);
   return 0;
 }
 
@@ -196,7 +171,6 @@ _cleanup(void) {
   }
 
   os_system_linux_netlink_remove(&_rtnetlink_socket);
-  os_system_linux_netlink_remove(&_rtnetlink_event_socket);
 }
 
 /**
@@ -255,7 +229,8 @@ os_routing_linux_set(struct os_route *route, bool set, bool del_similar) {
   if (set) {
     msg->nlmsg_flags |= NLM_F_CREATE | NLM_F_REPLACE;
     msg->nlmsg_type = RTM_NEWROUTE;
-  } else {
+  }
+  else {
     msg->nlmsg_type = RTM_DELROUTE;
 
     os_rt.p.protocol = 0;
@@ -270,15 +245,13 @@ os_routing_linux_set(struct os_route *route, bool set, bool del_similar) {
     }
   }
 
-  if (netaddr_is_unspec(&os_rt.p.gw)
-      && netaddr_get_address_family(&os_rt.p.key.dst) == AF_INET
-      && netaddr_get_prefix_length(&os_rt.p.key.dst) == netaddr_get_maxprefix(&os_rt.p.key.dst)) {
+  if (netaddr_is_unspec(&os_rt.p.gw) && netaddr_get_address_family(&os_rt.p.key.dst) == AF_INET &&
+      netaddr_get_prefix_length(&os_rt.p.key.dst) == netaddr_get_maxprefix(&os_rt.p.key.dst)) {
     /* use destination as gateway, to 'force' linux kernel to do proper source address selection */
     memcpy(&os_rt.p.gw, &os_rt.p.key.dst, sizeof(os_rt.p.gw));
   }
 
-  OONF_DEBUG(LOG_OS_ROUTING, "%sset route: %s", set ? "" : "re",
-      os_routing_to_string(&rbuf, &os_rt.p));
+  OONF_DEBUG(LOG_OS_ROUTING, "%sset route: %s", set ? "" : "re", os_routing_to_string(&rbuf, &os_rt.p));
 
   if (_routing_set(msg, &os_rt, scope)) {
     return -1;
@@ -291,7 +264,9 @@ os_routing_linux_set(struct os_route *route, bool set, bool del_similar) {
     route->_internal.nl_seq = seq;
     route->_internal._node.key = &route->_internal.nl_seq;
 
-    assert (!avl_is_node_added(&route->_internal._node));
+    OONF_ASSERT(!avl_is_node_added(&route->_internal._node),
+                LOG_OS_ROUTING, "route %s is already in feedback list!",
+                os_routing_to_string(&rbuf, &os_rt.p));
     avl_insert(&_rtnetlink_feedback, &route->_internal._node);
   }
   return 0;
@@ -309,7 +284,7 @@ os_routing_linux_query(struct os_route *route) {
   struct rtgenmsg *rt_gen;
   int seq;
 
-  assert (route->cb_finished != NULL && route->cb_get != NULL);
+  OONF_ASSERT(route->cb_finished != NULL && route->cb_get != NULL, LOG_OS_ROUTING, "illegal route query");
   memset(buffer, 0, sizeof(buffer));
 
   /* get pointers for netlink message */
@@ -405,12 +380,11 @@ _routing_finished(struct os_route *route, int error) {
  * Initiatize the an netlink routing message
  * @param msg pointer to netlink message header
  * @param route data to be added to the netlink message
- * @param scope scope of route to be set/removed
+ * @param rt_scope scope of route to be set/removed
  * @return -1 if an error happened, 0 otherwise
  */
 static int
-_routing_set(struct nlmsghdr *msg, struct os_route *route,
-    unsigned char rt_scope) {
+_routing_set(struct nlmsghdr *msg, struct os_route *route, unsigned char rt_scope) {
   struct rtmsg *rt_msg;
   size_t i;
 
@@ -419,27 +393,26 @@ _routing_set(struct nlmsghdr *msg, struct os_route *route,
     route->p.family = netaddr_get_address_family(&route->p.key.dst);
   }
   if (netaddr_get_address_family(&route->p.gw) != AF_UNSPEC) {
-    if (route->p.family  != AF_UNSPEC
-        && route->p.family  != netaddr_get_address_family(&route->p.gw)) {
+    if (route->p.family != AF_UNSPEC && route->p.family != netaddr_get_address_family(&route->p.gw)) {
       return -1;
     }
-    route->p.family  = netaddr_get_address_family(&route->p.gw);
+    route->p.family = netaddr_get_address_family(&route->p.gw);
   }
   if (netaddr_get_address_family(&route->p.src_ip) != AF_UNSPEC) {
-    if (route->p.family  != AF_UNSPEC && route->p.family  != netaddr_get_address_family(&route->p.src_ip)) {
+    if (route->p.family != AF_UNSPEC && route->p.family != netaddr_get_address_family(&route->p.src_ip)) {
       return -1;
     }
-    route->p.family  = netaddr_get_address_family(&route->p.src_ip);
+    route->p.family = netaddr_get_address_family(&route->p.src_ip);
   }
 
-  if (route->p.family  == AF_UNSPEC) {
-    route->p.family  = AF_INET;
+  if (route->p.family == AF_UNSPEC) {
+    route->p.family = AF_INET;
   }
 
   /* initialize rtmsg payload */
   rt_msg = NLMSG_DATA(msg);
 
-  rt_msg->rtm_family = route->p.family ;
+  rt_msg->rtm_family = route->p.family;
   rt_msg->rtm_scope = rt_scope;
   rt_msg->rtm_protocol = route->p.protocol;
   rt_msg->rtm_table = route->p.table;
@@ -448,7 +421,7 @@ _routing_set(struct nlmsghdr *msg, struct os_route *route,
   rt_msg->rtm_type = RTN_UNICAST;
 
   /* set route type */
-  for (i=0; i<ARRAYSIZE(_type_translation); i++) {
+  for (i = 0; i < ARRAYSIZE(_type_translation); i++) {
     if (_type_translation[i].oonf == route->p.type) {
       rt_msg->rtm_type = _type_translation[i].os_linux;
       break;
@@ -458,8 +431,7 @@ _routing_set(struct nlmsghdr *msg, struct os_route *route,
   /* add attributes */
   if (netaddr_get_address_family(&route->p.src_ip) != AF_UNSPEC) {
     /* add src-ip */
-    if (os_system_linux_netlink_addnetaddr(&_rtnetlink_event_socket,
-        msg, RTA_PREFSRC, &route->p.src_ip)) {
+    if (os_system_linux_netlink_addnetaddr(&_rtnetlink_socket, msg, RTA_PREFSRC, &route->p.src_ip)) {
       return -1;
     }
   }
@@ -468,8 +440,7 @@ _routing_set(struct nlmsghdr *msg, struct os_route *route,
     rt_msg->rtm_flags |= RTNH_F_ONLINK;
 
     /* add gateway */
-    if (os_system_linux_netlink_addnetaddr(&_rtnetlink_event_socket,
-        msg, RTA_GATEWAY, &route->p.gw)) {
+    if (os_system_linux_netlink_addnetaddr(&_rtnetlink_socket, msg, RTA_GATEWAY, &route->p.gw)) {
       return -1;
     }
   }
@@ -478,35 +449,32 @@ _routing_set(struct nlmsghdr *msg, struct os_route *route,
     rt_msg->rtm_dst_len = netaddr_get_prefix_length(&route->p.key.dst);
 
     /* add destination */
-    if (os_system_linux_netlink_addnetaddr(&_rtnetlink_event_socket,
-        msg, RTA_DST, &route->p.key.dst)) {
+    if (os_system_linux_netlink_addnetaddr(&_rtnetlink_socket, msg, RTA_DST, &route->p.key.dst)) {
       return -1;
     }
   }
 
-  if (netaddr_get_address_family(&route->p.key.src) == AF_INET6
-      && netaddr_get_prefix_length(&route->p.key.src) != 0) {
+  if (netaddr_get_address_family(&route->p.key.src) == AF_INET6 && netaddr_get_prefix_length(&route->p.key.src) != 0) {
     rt_msg->rtm_src_len = netaddr_get_prefix_length(&route->p.key.src);
 
     /* add source-specific routing prefix */
-    if (os_system_linux_netlink_addnetaddr(&_rtnetlink_event_socket,
-        msg, RTA_SRC, &route->p.key.src)) {
+    if (os_system_linux_netlink_addnetaddr(&_rtnetlink_socket, msg, RTA_SRC, &route->p.key.src)) {
       return -1;
     }
   }
 
   if (route->p.metric != -1) {
     /* add metric */
-    if (os_system_linux_netlink_addreq(&_rtnetlink_event_socket,
-        msg, RTA_PRIORITY, &route->p.metric, sizeof(route->p.metric))) {
+    if (os_system_linux_netlink_addreq(
+          &_rtnetlink_socket, msg, RTA_PRIORITY, &route->p.metric, sizeof(route->p.metric))) {
       return -1;
     }
   }
 
   if (route->p.if_index) {
     /* add interface*/
-    if (os_system_linux_netlink_addreq(&_rtnetlink_event_socket,
-        msg, RTA_OIF, &route->p.if_index, sizeof(route->p.if_index))) {
+    if (os_system_linux_netlink_addreq(
+          &_rtnetlink_socket, msg, RTA_OIF, &route->p.if_index, sizeof(route->p.if_index))) {
       return -1;
     }
   }
@@ -528,10 +496,11 @@ _routing_parse_nlmsg(struct os_route *route, struct nlmsghdr *msg) {
   size_t i;
 
   rt_msg = NLMSG_DATA(msg);
-  rt_attr = (struct rtattr *) RTM_RTA(rt_msg);
+  rt_attr = (struct rtattr *)RTM_RTA(rt_msg);
   rt_len = RTM_PAYLOAD(msg);
 
   if ((rt_msg->rtm_flags & RTM_F_CLONED) != 0) {
+    OONF_DEBUG(LOG_OS_ROUTING, "Received a cloned route");
     /* ignore cloned route events by returning the wildcard route */
     return 1;
   }
@@ -543,38 +512,38 @@ _routing_parse_nlmsg(struct os_route *route, struct nlmsghdr *msg) {
   route->p.family = rt_msg->rtm_family;
 
   if (route->p.family != AF_INET && route->p.family != AF_INET6) {
+    OONF_WARN(LOG_OS_ROUTING, "Got illegal route address family: %d", route->p.family);
     return -1;
   }
 
   /* get route type */
   route->p.type = OS_ROUTE_UNDEFINED;
-  for (i=0; i<ARRAYSIZE(_type_translation); i++) {
+  for (i = 0; i < ARRAYSIZE(_type_translation); i++) {
     if (rt_msg->rtm_type == _type_translation[i].os_linux) {
       route->p.type = _type_translation[i].oonf;
       break;
     }
   }
   if (route->p.type == OS_ROUTE_UNDEFINED) {
-    OONF_WARN(LOG_OS_ROUTING, "Got route type: %u", rt_msg->rtm_type);
-    return -1;
+    OONF_DEBUG(LOG_OS_ROUTING, "Got route type: %u", rt_msg->rtm_type);
+    return 2;
   }
 
-  for(; RTA_OK(rt_attr, rt_len); rt_attr = RTA_NEXT(rt_attr,rt_len)) {
-    switch(rt_attr->rta_type) {
+  for (; RTA_OK(rt_attr, rt_len); rt_attr = RTA_NEXT(rt_attr, rt_len)) {
+    switch (rt_attr->rta_type) {
       case RTA_PREFSRC:
-        netaddr_from_binary(&route->p.src_ip, RTA_DATA(rt_attr), RTA_PAYLOAD(rt_attr),
-            rt_msg->rtm_family);
+        netaddr_from_binary(&route->p.src_ip, RTA_DATA(rt_attr), RTA_PAYLOAD(rt_attr), rt_msg->rtm_family);
         break;
       case RTA_GATEWAY:
         netaddr_from_binary(&route->p.gw, RTA_DATA(rt_attr), RTA_PAYLOAD(rt_attr), rt_msg->rtm_family);
         break;
       case RTA_DST:
-        netaddr_from_binary_prefix(&route->p.key.dst, RTA_DATA(rt_attr), RTA_PAYLOAD(rt_attr),
-            rt_msg->rtm_family, rt_msg->rtm_dst_len);
+        netaddr_from_binary_prefix(
+          &route->p.key.dst, RTA_DATA(rt_attr), RTA_PAYLOAD(rt_attr), rt_msg->rtm_family, rt_msg->rtm_dst_len);
         break;
       case RTA_SRC:
-        netaddr_from_binary_prefix(&route->p.key.src, RTA_DATA(rt_attr),
-            RTA_PAYLOAD(rt_attr), rt_msg->rtm_family, rt_msg->rtm_src_len);
+        netaddr_from_binary_prefix(
+          &route->p.key.src, RTA_DATA(rt_attr), RTA_PAYLOAD(rt_attr), rt_msg->rtm_family, rt_msg->rtm_src_len);
         break;
       case RTA_PRIORITY:
         memcpy(&route->p.metric, RTA_DATA(rt_attr), sizeof(route->p.metric));
@@ -588,8 +557,8 @@ _routing_parse_nlmsg(struct os_route *route, struct nlmsghdr *msg) {
   }
 
   if (netaddr_get_address_family(&route->p.key.dst) == AF_UNSPEC) {
-    memcpy(&route->p.key.dst, route->p.family == AF_INET ? &NETADDR_IPV4_ANY : &NETADDR_IPV6_ANY,
-        sizeof(route->p.key.dst));
+    memcpy(
+      &route->p.key.dst, route->p.family == AF_INET ? &NETADDR_IPV4_ANY : &NETADDR_IPV6_ANY, sizeof(route->p.key.dst));
     netaddr_set_prefix_length(&route->p.key.dst, rt_msg->rtm_dst_len);
   }
   return 0;
@@ -606,23 +575,23 @@ _match_routes(struct os_route *filter, struct os_route *route) {
   if (filter->p.family != AF_UNSPEC && filter->p.family != route->p.family) {
     return false;
   }
-  if (netaddr_get_address_family(&filter->p.src_ip) != AF_UNSPEC
-      && memcmp(&filter->p.src_ip, &route->p.src_ip, sizeof(filter->p.src_ip)) != 0) {
+  if (netaddr_get_address_family(&filter->p.src_ip) != AF_UNSPEC &&
+      memcmp(&filter->p.src_ip, &route->p.src_ip, sizeof(filter->p.src_ip)) != 0) {
     return false;
   }
   if (filter->p.type != OS_ROUTE_UNDEFINED && filter->p.type != route->p.type) {
     return false;
   }
-  if (netaddr_get_address_family(&filter->p.gw) != AF_UNSPEC
-      && memcmp(&filter->p.gw, &route->p.gw, sizeof(filter->p.gw)) != 0) {
+  if (netaddr_get_address_family(&filter->p.gw) != AF_UNSPEC &&
+      memcmp(&filter->p.gw, &route->p.gw, sizeof(filter->p.gw)) != 0) {
     return false;
   }
-  if (netaddr_get_address_family(&filter->p.key.dst) != AF_UNSPEC
-      && memcmp(&filter->p.key.dst, &route->p.key.dst, sizeof(filter->p.key.dst)) != 0) {
+  if (netaddr_get_address_family(&filter->p.key.dst) != AF_UNSPEC &&
+      memcmp(&filter->p.key.dst, &route->p.key.dst, sizeof(filter->p.key.dst)) != 0) {
     return false;
   }
-  if (netaddr_get_address_family(&filter->p.key.src) != AF_UNSPEC
-      && memcmp(&filter->p.key.src, &route->p.key.src, sizeof(filter->p.key.src)) != 0) {
+  if (netaddr_get_address_family(&filter->p.key.src) != AF_UNSPEC &&
+      memcmp(&filter->p.key.src, &route->p.key.src, sizeof(filter->p.key.src)) != 0) {
     return false;
   }
   if (filter->p.metric != -1 && filter->p.metric != route->p.metric) {
@@ -639,15 +608,19 @@ _match_routes(struct os_route *filter, struct os_route *route) {
 
 /**
  * Handle incoming rtnetlink messages
- * @param msg
+ * @param msg netlink message including header
  */
 static void
 _cb_rtnetlink_message(struct nlmsghdr *msg) {
+  struct os_route_listener *listener;
   struct os_route *filter;
   struct os_route rt;
   int result;
 
-  OONF_DEBUG(LOG_OS_ROUTING, "Got message: %d %d", msg->nlmsg_seq, msg->nlmsg_type);
+#ifdef OONF_LOG_DEBUG_INFO
+  struct os_route_str rbuf;
+#endif
+  OONF_DEBUG(LOG_OS_ROUTING, "Got message: %d %d 0x%04x", msg->nlmsg_seq, msg->nlmsg_type, msg->nlmsg_flags);
 
   if (msg->nlmsg_type != RTM_NEWROUTE && msg->nlmsg_type != RTM_DELROUTE) {
     return;
@@ -655,53 +628,41 @@ _cb_rtnetlink_message(struct nlmsghdr *msg) {
 
   if ((result = _routing_parse_nlmsg(&rt, msg))) {
     if (result < 0) {
-      OONF_WARN(LOG_OS_ROUTING, "Error while processing route reply");
+      OONF_WARN(LOG_OS_ROUTING, "Error while processing route reply (result %d)", result);
     }
     return;
   }
 
-  /* check for feedback for ongoing route commands */
-  filter = avl_find_element(&_rtnetlink_feedback, &msg->nlmsg_seq, filter, _internal._node);
+  OONF_DEBUG(LOG_OS_ROUTING, "Content: %s", os_routing_to_string(&rbuf, &rt.p));
+
+  /*
+   * sometimes netlink messages from the kernel have a (random?) sequence number, so
+   * we deliver everything with seq==0 and with an unregistered sequence number
+   * to the attached listeners now
+   */
+  filter = NULL;
+  if (msg->nlmsg_seq != 0) {
+    /* check for feedback for ongoing route commands */
+    filter = avl_find_element(&_rtnetlink_feedback, &msg->nlmsg_seq, filter, _internal._node);
+  }
+
   if (filter) {
     if (filter->cb_get != NULL && _match_routes(filter, &rt)) {
       filter->cb_get(filter, &rt);
     }
   }
-}
-
-/**
- * Handle incoming rtnetlink messages
- * @param msg
- */
-static void
-_cb_rtnetlink_event_message(struct nlmsghdr *msg) {
-  struct os_route_listener *listener;
-  struct os_route rt;
-  int result;
-
-  OONF_DEBUG(LOG_OS_ROUTING, "Got event message: %d %d", msg->nlmsg_seq, msg->nlmsg_type);
-
-  if (msg->nlmsg_type != RTM_NEWROUTE && msg->nlmsg_type != RTM_DELROUTE) {
-    return;
-  }
-
-  if ((result = _routing_parse_nlmsg(&rt, msg))) {
-    if (result < 0) {
-      OONF_WARN(LOG_OS_ROUTING, "Error while processing route reply");
+  else {
+    /* send route events to listeners */
+    list_for_each_element(&_rtnetlink_listener, listener, _internal._node) {
+      listener->cb_get(&rt, msg->nlmsg_type == RTM_NEWROUTE);
     }
-    return;
-  }
-
-  /* send route events to listeners */
-  list_for_each_element(&_rtnetlink_listener, listener, _internal._node) {
-    listener->cb_get(&rt, msg->nlmsg_type == RTM_NEWROUTE);
   }
 }
 
 /**
  * Handle feedback from netlink socket
- * @param seq
- * @param error
+ * @param seq sequence number of netlink response
+ * @param err error value
  */
 static void
 _cb_rtnetlink_error(uint32_t seq, int err) {
@@ -710,18 +671,15 @@ _cb_rtnetlink_error(uint32_t seq, int err) {
   struct os_route_str rbuf;
 #endif
 
-  /* transform into errno number */
   route = avl_find_element(&_rtnetlink_feedback, &seq, route, _internal._node);
   if (route) {
-    OONF_DEBUG(LOG_OS_ROUTING, "Route seqno %u failed: %s (%d) %s",
-        seq, strerror(err), err,
-        os_routing_to_string(&rbuf, &route->p));
+    OONF_DEBUG(LOG_OS_ROUTING, "Route seqno %u failed: %s (%d) %s", seq, strerror(err), err,
+      os_routing_to_string(&rbuf, &route->p));
 
     _routing_finished(route, err);
   }
   else {
-    OONF_DEBUG(LOG_OS_ROUTING, "Unknown route with seqno %u failed: %s (%d)",
-        seq, strerror(err), err);
+    OONF_DEBUG(LOG_OS_ROUTING, "Unknown route with seqno %u failed: %s (%d)", seq, strerror(err), err);
   }
 }
 
@@ -741,7 +699,7 @@ _cb_rtnetlink_timeout(void) {
 
 /**
  * Handle done from multipart netlink messages
- * @param seq
+ * @param seq netlink sequence number
  */
 static void
 _cb_rtnetlink_done(uint32_t seq) {
@@ -754,8 +712,7 @@ _cb_rtnetlink_done(uint32_t seq) {
 
   route = avl_find_element(&_rtnetlink_feedback, &seq, route, _internal._node);
   if (route) {
-    OONF_DEBUG(LOG_OS_ROUTING, "Route %s with seqno %u done",
-        os_routing_to_string(&rbuf, &route->p), seq);
+    OONF_DEBUG(LOG_OS_ROUTING, "Route %s with seqno %u done", os_routing_to_string(&rbuf, &route->p), seq);
     _routing_finished(route, 0);
   }
 }
