@@ -38,18 +38,27 @@
  * the copyright holders.
  *
  */
+
+/**
+ * @file
+ */
 #include <assert.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include "common/common_types.h"
 #include "common/avl.h"
 #include "common/avl_comp.h"
 #include "common/bitmap256.h"
-#include "rfc5444_reader.h"
+#include "common/common_types.h"
 #include "rfc5444_api_config.h"
+#include "rfc5444_reader.h"
 
 #if DISALLOW_CONSUMER_CONTEXT_DROP == true
+/**
+ * Helper function to allow to switch consumer context drop on/off
+ * @param value value if context drop is allowed
+ * @param def value if context drop is not allowed
+ */
 #define RFC5444_CONSUMER_DROP_ONLY(value, def) (def)
 #else
 #define RFC5444_CONSUMER_DROP_ONLY(value, def) (value)
@@ -58,28 +67,28 @@
 static int _consumer_avl_comp(const void *k1, const void *k2);
 static uint16_t _calc_tlvconsumer_intorder(struct rfc5444_reader_tlvblock_consumer_entry *entry);
 static uint16_t _calc_tlvblock_intorder(struct rfc5444_reader_tlvblock_entry *entry);
-static int _compare_tlvtypes(struct rfc5444_reader_tlvblock_entry *tlv,
-    struct rfc5444_reader_tlvblock_consumer_entry *entry);
-static uint8_t _rfc5444_get_u8(uint8_t **ptr, uint8_t *end, enum rfc5444_result *result);
-static uint16_t _rfc5444_get_u16(uint8_t **ptr, uint8_t *end, enum rfc5444_result *result);
+static int _compare_tlvtypes(
+  struct rfc5444_reader_tlvblock_entry *tlv, struct rfc5444_reader_tlvblock_consumer_entry *entry);
+static uint8_t _rfc5444_get_u8(const uint8_t **ptr, const uint8_t *end, enum rfc5444_result *result);
+static uint16_t _rfc5444_get_u16(const uint8_t **ptr, const uint8_t *end, enum rfc5444_result *result);
 static void _free_tlvblock(struct rfc5444_reader *parser, struct avl_tree *entries);
-static int _parse_tlv(struct rfc5444_reader_tlvblock_entry *entry, uint8_t **ptr,
-    uint8_t *eob, uint8_t addr_count);
-static int _parse_tlvblock(struct rfc5444_reader *parser,
-    struct avl_tree *tlvblock, uint8_t **ptr, uint8_t *eob, uint8_t addr_count);
+static int _parse_tlv(
+  struct rfc5444_reader_tlvblock_entry *entry, const uint8_t **ptr, const uint8_t *eob, uint8_t addr_count);
+static int _parse_tlvblock(struct rfc5444_reader *parser, struct avl_tree *tlvblock, const uint8_t **ptr,
+  const uint8_t *eob, uint8_t addr_count);
 static int _schedule_tlvblock(struct rfc5444_reader_tlvblock_consumer *consumer,
-    struct rfc5444_reader_tlvblock_context *context, struct avl_tree *entries, uint8_t idx);
+  struct rfc5444_reader_tlvblock_context *context, struct avl_tree *entries, uint8_t idx);
 static int _parse_addrblock(struct rfc5444_reader_addrblock_entry *addr_entry,
-    struct rfc5444_reader_tlvblock_context *tlv_context, uint8_t **ptr, uint8_t *eob);
-static int _handle_message(struct rfc5444_reader *parser,
-    struct rfc5444_reader_tlvblock_context *tlv_context, uint8_t **ptr, uint8_t *eob);
-static struct rfc5444_reader_tlvblock_consumer *_add_consumer(
-    struct rfc5444_reader_tlvblock_consumer *, struct avl_tree *consumer_tree,
-    struct rfc5444_reader_tlvblock_consumer_entry *entries, int entrycount);
-static void _free_consumer(struct avl_tree *consumer_tree,
-    struct rfc5444_reader_tlvblock_consumer *consumer);
+  struct rfc5444_reader_tlvblock_context *tlv_context, const uint8_t **ptr, const uint8_t *eob);
+static int _handle_message(struct rfc5444_reader *parser, struct rfc5444_reader_tlvblock_context *tlv_context,
+  const uint8_t **ptr, const uint8_t *eob);
+static struct rfc5444_reader_tlvblock_consumer *_add_consumer(struct rfc5444_reader_tlvblock_consumer *,
+  struct avl_tree *consumer_tree, struct rfc5444_reader_tlvblock_consumer_entry *entries, int entrycount);
+static void _free_consumer(struct avl_tree *consumer_tree, struct rfc5444_reader_tlvblock_consumer *consumer);
 static struct rfc5444_reader_addrblock_entry *_malloc_addrblock_entry(void);
 static struct rfc5444_reader_tlvblock_entry *_malloc_tlvblock_entry(void);
+static void _free_addrblock_entry(struct rfc5444_reader_addrblock_entry *entry);
+static void _free_tlvblock_entry(struct rfc5444_reader_tlvblock_entry *entry);
 
 static uint8_t rfc5444_get_pktversion(uint8_t v);
 
@@ -98,9 +107,9 @@ rfc5444_reader_init(struct rfc5444_reader *context) {
     context->malloc_tlvblock_entry = _malloc_tlvblock_entry;
 
   if (context->free_addrblock_entry == NULL)
-    context->free_addrblock_entry = free;
+    context->free_addrblock_entry = _free_addrblock_entry;
   if (context->free_tlvblock_entry == NULL)
-    context->free_tlvblock_entry = free;
+    context->free_tlvblock_entry = _free_tlvblock_entry;
 }
 
 /**
@@ -122,11 +131,12 @@ rfc5444_reader_cleanup(struct rfc5444_reader *context) {
  * @return RFC5444_OKAY (0) if successful, RFC5444_... otherwise
  */
 enum rfc5444_result
-rfc5444_reader_handle_packet(struct rfc5444_reader *parser, uint8_t *buffer, size_t length) {
+rfc5444_reader_handle_packet(struct rfc5444_reader *parser, const uint8_t *buffer, size_t length)
+{
   struct rfc5444_reader_tlvblock_context context;
   struct avl_tree entries;
   struct rfc5444_reader_tlvblock_consumer *consumer, *last_started;
-  uint8_t *ptr, *eob;
+  const uint8_t *ptr, *eob;
   bool has_tlv;
   uint8_t first_byte;
   enum rfc5444_result result = RFC5444_OKAY;
@@ -149,7 +159,7 @@ rfc5444_reader_handle_packet(struct rfc5444_reader *parser, uint8_t *buffer, siz
   context.pkt_version = rfc5444_get_pktversion(first_byte);
   context.pkt_flags = first_byte & RFC5444_PKT_FLAGMASK;
 
-  if (context.pkt_version!= 0) {
+  if (context.pkt_version != 0) {
     /*
      * bad packet version, do not jump to cleanup_parse packet because
      * we have not allocated any resources at this point
@@ -201,7 +211,7 @@ rfc5444_reader_handle_packet(struct rfc5444_reader *parser, uint8_t *buffer, siz
 #if DISALLOW_CONSUMER_CONTEXT_DROP == false
       result =
 #endif
-          consumer->start_callback(&context);
+        consumer->start_callback(&context);
 #if DISALLOW_CONSUMER_CONTEXT_DROP == false
       if (result != RFC5444_OKAY) {
         goto cleanup_parse_packet;
@@ -214,7 +224,7 @@ rfc5444_reader_handle_packet(struct rfc5444_reader *parser, uint8_t *buffer, siz
 #if DISALLOW_CONSUMER_CONTEXT_DROP == false
       result =
 #endif
-          _schedule_tlvblock(consumer, &context, &entries, 0);
+        _schedule_tlvblock(consumer, &context, &entries, 0);
 #if DISALLOW_CONSUMER_CONTEXT_DROP == false
       if (result != RFC5444_OKAY) {
         goto cleanup_parse_packet;
@@ -260,9 +270,8 @@ cleanup_parse_packet:
  * @param entrycount number of elements in array
  */
 void
-rfc5444_reader_add_packet_consumer(struct rfc5444_reader *parser,
-    struct rfc5444_reader_tlvblock_consumer *consumer,
-    struct rfc5444_reader_tlvblock_consumer_entry *entries, size_t entrycount) {
+rfc5444_reader_add_packet_consumer(struct rfc5444_reader *parser, struct rfc5444_reader_tlvblock_consumer *consumer,
+  struct rfc5444_reader_tlvblock_consumer_entry *entries, size_t entrycount) {
   _add_consumer(consumer, &parser->packet_consumer, entries, entrycount);
 }
 
@@ -275,9 +284,8 @@ rfc5444_reader_add_packet_consumer(struct rfc5444_reader *parser,
  * @param entrycount number of elements in array
  */
 void
-rfc5444_reader_add_message_consumer(struct rfc5444_reader *parser,
-    struct rfc5444_reader_tlvblock_consumer *consumer,
-    struct rfc5444_reader_tlvblock_consumer_entry *entries, size_t entrycount) {
+rfc5444_reader_add_message_consumer(struct rfc5444_reader *parser, struct rfc5444_reader_tlvblock_consumer *consumer,
+  struct rfc5444_reader_tlvblock_consumer_entry *entries, size_t entrycount) {
   _add_consumer(consumer, &parser->message_consumer, entries, entrycount);
 }
 
@@ -287,9 +295,9 @@ rfc5444_reader_add_message_consumer(struct rfc5444_reader *parser,
  * @param consumer pointer to tlvblock consumer
  */
 void
-rfc5444_reader_remove_packet_consumer(struct rfc5444_reader *parser,
-    struct rfc5444_reader_tlvblock_consumer *consumer) {
-  assert (!consumer->addrblock_consumer && consumer->msg_id == 0);
+rfc5444_reader_remove_packet_consumer(
+  struct rfc5444_reader *parser, struct rfc5444_reader_tlvblock_consumer *consumer) {
+  assert(!consumer->addrblock_consumer && consumer->msg_id == 0);
   _free_consumer(&parser->packet_consumer, consumer);
 }
 
@@ -299,8 +307,8 @@ rfc5444_reader_remove_packet_consumer(struct rfc5444_reader *parser,
  * @param consumer pointer to tlvblock consumer
  */
 void
-rfc5444_reader_remove_message_consumer(struct rfc5444_reader *parser,
-    struct rfc5444_reader_tlvblock_consumer *consumer) {
+rfc5444_reader_remove_message_consumer(
+  struct rfc5444_reader *parser, struct rfc5444_reader_tlvblock_consumer *consumer) {
   _free_consumer(&parser->message_consumer, consumer);
 }
 
@@ -349,10 +357,9 @@ _calc_tlvblock_intorder(struct rfc5444_reader_tlvblock_entry *entry) {
 }
 
 /**
- * Checks if two internal types have the same tlv type
- * @param int_type1 first internal type
- * @param int_type2 second internal type
- * @param match_ext true if extension type is relevant
+ * Checks if a tlvblock consumer entry to a tlvblock entry
+ * @param tlv tlvlock entry, might be NULL
+ * @param entry tlvblock consumer entry, might be NULL
  * @return <0 if type1<type2, ==0 if type1==type2, >0 if type1>type2
  */
 static int
@@ -383,7 +390,7 @@ _compare_tlvtypes(struct rfc5444_reader_tlvblock_entry *tlv, struct rfc5444_read
  * @return uint8_t value of the next byte
  */
 static uint8_t
-_rfc5444_get_u8(uint8_t **ptr, uint8_t *end, enum rfc5444_result *error) {
+_rfc5444_get_u8(const uint8_t **ptr, const uint8_t *end, enum rfc5444_result *error) {
   uint8_t result;
   if (*error != RFC5444_OKAY) {
     return 0;
@@ -408,9 +415,8 @@ _rfc5444_get_u8(uint8_t **ptr, uint8_t *end, enum rfc5444_result *error) {
  * @return uint16_t value of the next word (network byte order)
  */
 static uint16_t
-_rfc5444_get_u16(uint8_t **ptr, uint8_t *end, enum rfc5444_result *error) {
-  return ((uint16_t)_rfc5444_get_u8(ptr, end, error) << 8)
-      | (uint16_t)_rfc5444_get_u8(ptr, end, error);
+_rfc5444_get_u16(const uint8_t **ptr, const uint8_t *end, enum rfc5444_result *error) {
+  return ((uint16_t)_rfc5444_get_u8(ptr, end, error) << 8) | (uint16_t)_rfc5444_get_u8(ptr, end, error);
 }
 
 /**
@@ -438,7 +444,7 @@ _free_tlvblock(struct rfc5444_reader *parser, struct avl_tree *entries) {
  * @return -1 if an error happened, 0 otherwise
  */
 static enum rfc5444_result
-_parse_tlv(struct rfc5444_reader_tlvblock_entry *entry, uint8_t **ptr, uint8_t *eob, uint8_t addr_count) {
+_parse_tlv(struct rfc5444_reader_tlvblock_entry *entry, const uint8_t **ptr, const uint8_t *eob, uint8_t addr_count) {
   enum rfc5444_result result = RFC5444_OKAY;
   uint8_t masked, count;
 
@@ -462,7 +468,7 @@ _parse_tlv(struct rfc5444_reader_tlvblock_entry *entry, uint8_t **ptr, uint8_t *
   masked = entry->flags & (RFC5444_TLV_FLAG_SINGLE_IDX | RFC5444_TLV_FLAG_MULTI_IDX);
   if (masked == 0) {
     entry->index1 = 0;
-    entry->index2 = addr_count-1;
+    entry->index2 = addr_count - 1;
   }
   else if (masked == RFC5444_TLV_FLAG_SINGLE_IDX) {
     entry->index1 = entry->index2 = _rfc5444_get_u8(ptr, eob, &result);
@@ -472,7 +478,14 @@ _parse_tlv(struct rfc5444_reader_tlvblock_entry *entry, uint8_t **ptr, uint8_t *
     entry->index2 = _rfc5444_get_u8(ptr, eob, &result);
   }
   else {
-    result = RFC5444_BAD_TLV_IDXFLAGS;
+    *ptr = eob;
+    return RFC5444_BAD_TLV_IDXFLAGS;
+  }
+
+  /* consistency check for index fields */
+  if (addr_count > 0 && (entry->index1 >= addr_count || entry->index2 >= addr_count)) {
+    *ptr = eob;
+    return RFC5444_BAD_TLV_INDEX;
   }
 
   /* check for length field */
@@ -487,7 +500,8 @@ _parse_tlv(struct rfc5444_reader_tlvblock_entry *entry, uint8_t **ptr, uint8_t *
     entry->length = _rfc5444_get_u16(ptr, eob, &result);
   }
   else {
-    result = RFC5444_BAD_TLV_VALUEFLAGS;
+    *ptr = eob;
+    return RFC5444_BAD_TLV_VALUEFLAGS;
   }
 
   /* check for multivalue tlv field */
@@ -495,7 +509,8 @@ _parse_tlv(struct rfc5444_reader_tlvblock_entry *entry, uint8_t **ptr, uint8_t *
 
   /* not enough bytes left ? */
   if (*ptr + entry->length > eob) {
-    result = RFC5444_END_OF_BUFFER;
+    *ptr = eob;
+    return RFC5444_END_OF_BUFFER;
   }
   if (result != RFC5444_OKAY) {
     *ptr = eob;
@@ -541,12 +556,12 @@ _parse_tlv(struct rfc5444_reader_tlvblock_entry *entry, uint8_t **ptr, uint8_t *
  *   packet tlv * @return -1 if an error happened, 0 otherwise
  */
 static enum rfc5444_result
-_parse_tlvblock(struct rfc5444_reader *parser,
-    struct avl_tree *tlvblock, uint8_t **ptr, uint8_t *eob, uint8_t addr_count) {
+_parse_tlvblock(struct rfc5444_reader *parser, struct avl_tree *tlvblock, const uint8_t **ptr, const uint8_t *eob,
+  uint8_t addr_count) {
   enum rfc5444_result result = RFC5444_OKAY;
   struct rfc5444_reader_tlvblock_entry *tlv1 = NULL;
   struct rfc5444_reader_tlvblock_entry entry;
-  uint8_t *end;
+  const uint8_t *end;
 
   /* get length of TLV block */
   end = (*ptr) + 2;
@@ -579,7 +594,7 @@ _parse_tlvblock(struct rfc5444_reader *parser,
     }
 
     /* copy TLV block entry into allocated memory */
-    memcpy (tlv1, &entry, sizeof(entry));
+    memcpy(tlv1, &entry, sizeof(entry));
 
     /* put into sorted list */
     tlv1->node.key = &tlv1->_order;
@@ -598,7 +613,7 @@ cleanup_parse_tlvblock:
  * @param consumer pointer to first consumer for this message type
  * @param context pointer to context for tlv block
  * @param entries pointer avl_tree of tlv block entries
- * @param index of current address inside the addressblock, 0 for message tlv block
+ * @param idx of current address inside the addressblock, 0 for message tlv block
  * @return RFC5444_TLV_DROP_ADDRESS if the current address should
  *   be dropped for later consumers, RFC5444_TLV_DROP_CONTEXT if
  *   the complete message/package should be dropped for
@@ -606,7 +621,7 @@ cleanup_parse_tlvblock:
  */
 static enum rfc5444_result
 _schedule_tlvblock(struct rfc5444_reader_tlvblock_consumer *consumer, struct rfc5444_reader_tlvblock_context *context,
-    struct avl_tree *entries, uint8_t idx) {
+  struct avl_tree *entries, uint8_t idx) {
   struct rfc5444_reader_tlvblock_entry *tlv = NULL, *nexttlv = NULL;
   struct rfc5444_reader_tlvblock_consumer_entry *cons_entry;
   bool constraints_failed;
@@ -637,8 +652,8 @@ _schedule_tlvblock(struct rfc5444_reader_tlvblock_consumer *consumer, struct rfc
     bool index_match = false;
 
     if (tlv) {
-      index_match = RFC5444_CONSUMER_DROP_ONLY(!bitmap256_get(&tlv->int_drop_tlv, idx), true)
-          && idx >= tlv->index1 && idx <= tlv->index2;
+      index_match = RFC5444_CONSUMER_DROP_ONLY(!bitmap256_get(&tlv->int_drop_tlv, idx), true) && idx >= tlv->index1 &&
+                    idx <= tlv->index2;
     }
 
     /* check index for address blocks */
@@ -662,7 +677,7 @@ _schedule_tlvblock(struct rfc5444_reader_tlvblock_consumer *consumer, struct rfc
 #if DISALLOW_CONSUMER_CONTEXT_DROP == false
       result =
 #endif
-          consumer->tlv_callback(tlv, context);
+        consumer->tlv_callback(tlv, context);
 #if DISALLOW_CONSUMER_CONTEXT_DROP == false
       if (result == RFC5444_DROP_TLV) {
         /* mark dropped tlv */
@@ -684,8 +699,7 @@ _schedule_tlvblock(struct rfc5444_reader_tlvblock_consumer *consumer, struct rfc
 
       if (match) {
         if (cons_entry->match_length &&
-            (tlv->length < cons_entry->min_length
-                || tlv->length > cons_entry->max_length)) {
+            (tlv->length < cons_entry->min_length || tlv->length > cons_entry->max_length)) {
           constraints_failed = true;
         }
 
@@ -745,14 +759,14 @@ _schedule_tlvblock(struct rfc5444_reader_tlvblock_consumer *consumer, struct rfc
 #if DISALLOW_CONSUMER_CONTEXT_DROP == false
     result =
 #endif
-        consumer->block_callback(context);
+      consumer->block_callback(context);
   }
   else if (consumer->block_callback_failed_constraints != NULL && constraints_failed) {
     context->consumer = consumer;
 #if DISALLOW_CONSUMER_CONTEXT_DROP == false
     result =
 #endif
-        consumer->block_callback_failed_constraints(context);
+      consumer->block_callback_failed_constraints(context);
   }
 #if DISALLOW_CONSUMER_CONTEXT_DROP == false
   if (result == RFC5444_DROP_TLV) {
@@ -770,7 +784,7 @@ _schedule_tlvblock(struct rfc5444_reader_tlvblock_consumer *consumer, struct rfc
 #if DISALLOW_CONSUMER_CONTEXT_DROP == false
 cleanup_handle_tlvblock:
 #endif
-#if  DEBUG_CLEANUP == true
+#if DEBUG_CLEANUP == true
   list_for_each_element(&consumer->_consumer_list, cons_entry, _node) {
     cons_entry->tlv = NULL;
     cons_entry->drop = false;
@@ -791,8 +805,8 @@ cleanup_handle_tlvblock:
  * @return -1 if an error happened, 0 otherwise
  */
 static enum rfc5444_result
-_parse_addrblock(struct rfc5444_reader_addrblock_entry *addr_entry,
-    struct rfc5444_reader_tlvblock_context *tlv_context, uint8_t **ptr, uint8_t *eob) {
+_parse_addrblock(struct rfc5444_reader_addrblock_entry *addr_entry, struct rfc5444_reader_tlvblock_context *tlv_context,
+  const uint8_t **ptr, const uint8_t *eob) {
   enum rfc5444_result result = RFC5444_OKAY;
   uint8_t flags;
   uint8_t tail_len;
@@ -817,6 +831,10 @@ _parse_addrblock(struct rfc5444_reader_addrblock_entry *addr_entry,
   /* check for head flag */
   if ((flags & RFC5444_ADDR_FLAG_HEAD) != 0) {
     addr_entry->mid_start = _rfc5444_get_u8(ptr, eob, &result);
+    if (addr_entry->mid_start < 1 || addr_entry->mid_start >= RFC5444_MAX_ADDRLEN ||
+        addr_entry->mid_start >= tlv_context->addr_len) {
+      return RFC5444_BAD_ADDR_HEAD_LENGTH;
+    }
     if (*ptr + addr_entry->mid_start > eob) {
       /* not enough buffer for head */
       return RFC5444_END_OF_BUFFER;
@@ -835,6 +853,9 @@ _parse_addrblock(struct rfc5444_reader_addrblock_entry *addr_entry,
   }
   else if (masked == RFC5444_ADDR_FLAG_FULLTAIL) {
     tail_len = _rfc5444_get_u8(ptr, eob, &result);
+    if (tail_len < 1 || tail_len >= RFC5444_MAX_ADDRLEN || tail_len >= tlv_context->addr_len) {
+      return RFC5444_BAD_ADDR_TAIL_LENGTH;
+    }
     if (*ptr + tail_len > eob) {
       /* not enough buffer for head */
       return RFC5444_END_OF_BUFFER;
@@ -893,7 +914,7 @@ _parse_addrblock(struct rfc5444_reader_addrblock_entry *addr_entry,
  */
 static enum rfc5444_result
 schedule_msgtlv_consumer(struct rfc5444_reader_tlvblock_consumer *consumer,
-    struct rfc5444_reader_tlvblock_context *tlv_context, struct avl_tree *tlv_entries) {
+  struct rfc5444_reader_tlvblock_context *tlv_context, struct avl_tree *tlv_entries) {
   enum rfc5444_result result = RFC5444_OKAY;
   tlv_context->type = RFC5444_CONTEXT_MESSAGE;
 
@@ -904,7 +925,7 @@ schedule_msgtlv_consumer(struct rfc5444_reader_tlvblock_consumer *consumer,
 #if DISALLOW_CONSUMER_CONTEXT_DROP == false
     result =
 #endif
-        consumer->start_callback(tlv_context);
+      consumer->start_callback(tlv_context);
   }
 
   /* call consumer for message tlv block */
@@ -925,7 +946,7 @@ schedule_msgtlv_consumer(struct rfc5444_reader_tlvblock_consumer *consumer,
  */
 static enum rfc5444_result
 schedule_msgaddr_consumer(struct rfc5444_reader_tlvblock_consumer *consumer,
-    struct rfc5444_reader_tlvblock_context *tlv_context, struct list_entity *addr_head) {
+  struct rfc5444_reader_tlvblock_context *tlv_context, struct list_entity *addr_head) {
   struct rfc5444_reader_addrblock_entry *addr;
   enum rfc5444_result result = RFC5444_OKAY;
 
@@ -942,7 +963,7 @@ schedule_msgaddr_consumer(struct rfc5444_reader_tlvblock_consumer *consumer,
     tlv_context->addr_tlv_size = addr->addr_tlv_size;
 
     /* iterate over all addresses in block */
-    for (i=0; i<addr->num_addr; i++) {
+    for (i = 0; i < addr->num_addr; i++) {
       /* test if we should skip this address */
 #if DISALLOW_CONSUMER_CONTEXT_DROP == false
       if (bitmap256_get(&addr->dropAddr, i)) {
@@ -960,8 +981,7 @@ schedule_msgaddr_consumer(struct rfc5444_reader_tlvblock_consumer *consumer,
       else {
         plen = addr->prefixlen;
       }
-      netaddr_from_binary_prefix(&tlv_context->addr, addr->addr,
-          tlv_context->addr_len, 0, plen);
+      netaddr_from_binary_prefix(&tlv_context->addr, addr->addr, tlv_context->addr_len, 0, plen);
 
       /* remember index of address */
       tlv_context->addr_index = i;
@@ -973,7 +993,7 @@ schedule_msgaddr_consumer(struct rfc5444_reader_tlvblock_consumer *consumer,
 #if DISALLOW_CONSUMER_CONTEXT_DROP == false
         result =
 #endif
-            consumer->start_callback(tlv_context);
+          consumer->start_callback(tlv_context);
       }
 
       /* handle tlvblock callbacks */
@@ -1020,16 +1040,16 @@ schedule_msgaddr_consumer(struct rfc5444_reader_tlvblock_consumer *consumer,
  */
 static enum rfc5444_result
 schedule_end_message_cbs(struct rfc5444_reader_tlvblock_context *tlv_context,
-    struct rfc5444_reader_tlvblock_consumer *first, struct rfc5444_reader_tlvblock_consumer *last,
-    enum rfc5444_result result) {
+  struct rfc5444_reader_tlvblock_consumer *first, struct rfc5444_reader_tlvblock_consumer *last,
+  enum rfc5444_result result) {
   struct rfc5444_reader_tlvblock_consumer *consumer;
   enum rfc5444_result r;
 
   tlv_context->type = RFC5444_CONTEXT_MESSAGE;
 
   avl_for_element_range_reverse(first, last, consumer, _node) {
-    if (consumer->end_callback && !consumer->addrblock_consumer
-        && (consumer->default_msg_consumer || consumer->msg_id == tlv_context->msg_type)) {
+    if (consumer->end_callback && !consumer->addrblock_consumer &&
+        (consumer->default_msg_consumer || consumer->msg_id == tlv_context->msg_type)) {
       tlv_context->consumer = consumer;
       r = consumer->end_callback(tlv_context, result != RFC5444_OKAY);
       if (r > result) {
@@ -1051,13 +1071,13 @@ schedule_end_message_cbs(struct rfc5444_reader_tlvblock_context *tlv_context,
  * @return -1 if an error happened, 0 otherwise
  */
 static enum rfc5444_result
-_handle_message(struct rfc5444_reader *parser,
-    struct rfc5444_reader_tlvblock_context *tlv_context, uint8_t **ptr, uint8_t *eob) {
+_handle_message(struct rfc5444_reader *parser, struct rfc5444_reader_tlvblock_context *tlv_context, const uint8_t **ptr,
+  const uint8_t *eob) {
   struct avl_tree tlv_entries;
   struct rfc5444_reader_tlvblock_consumer *consumer, *same_order[2];
   struct list_entity addr_head;
   struct rfc5444_reader_addrblock_entry *addr, *safe;
-  uint8_t *start, *end = NULL;
+  const uint8_t *start, *end = NULL;
   uint8_t flags;
   uint16_t size;
 
@@ -1175,8 +1195,7 @@ _handle_message(struct rfc5444_reader *parser,
 #if DISALLOW_CONSUMER_CONTEXT_DROP == false
       result =
 #endif
-      schedule_end_message_cbs(tlv_context,
-          same_order[0], same_order[1], result);
+        schedule_end_message_cbs(tlv_context, same_order[0], same_order[1], result);
 #if DISALLOW_CONSUMER_CONTEXT_DROP == false
       if (result != RFC5444_OKAY) {
         goto cleanup_parse_message;
@@ -1189,13 +1208,13 @@ _handle_message(struct rfc5444_reader *parser,
 #if DISALLOW_CONSUMER_CONTEXT_DROP == false
       result =
 #endif
-      schedule_msgaddr_consumer(consumer, tlv_context, &addr_head);
+        schedule_msgaddr_consumer(consumer, tlv_context, &addr_head);
     }
     else {
 #if DISALLOW_CONSUMER_CONTEXT_DROP == false
       result =
 #endif
-      schedule_msgtlv_consumer(consumer, tlv_context, &tlv_entries);
+        schedule_msgtlv_consumer(consumer, tlv_context, &tlv_entries);
       if (same_order[0] == NULL) {
         same_order[0] = consumer;
       }
@@ -1214,8 +1233,7 @@ _handle_message(struct rfc5444_reader *parser,
 #if DISALLOW_CONSUMER_CONTEXT_DROP == false
     result =
 #endif
-    schedule_end_message_cbs(tlv_context,
-        same_order[0], same_order[1], result);
+      schedule_end_message_cbs(tlv_context, same_order[0], same_order[1], result);
 #if DISALLOW_CONSUMER_CONTEXT_DROP == false
     if (result != RFC5444_OKAY) {
       goto cleanup_parse_message;
@@ -1230,10 +1248,9 @@ cleanup_parse_message:
   /* handle message forwarding */
   if (
 #if DISALLOW_CONSUMER_CONTEXT_DROP == false
-      (result == RFC5444_OKAY || result == RFC5444_DROP_MSG_BUT_FORWARD) &&
+    (result == RFC5444_OKAY || result == RFC5444_DROP_MSG_BUT_FORWARD) &&
 #endif
-      !tlv_context->_do_not_forward
-      && parser->forward_message != NULL && tlv_context->has_hoplimit) {
+    !tlv_context->_do_not_forward && parser->forward_message != NULL && tlv_context->has_hoplimit) {
     /* check limit */
     if (tlv_context->hoplimit > 1) {
       /* forward message if callback is available */
@@ -1263,17 +1280,16 @@ cleanup_parse_message:
 /**
  * Add a tlvblock consumer to a linked list of consumers.
  * The list is kept sorted by the order of the consumers.
- * @param parser pointer to parser context
+ * @param consumer tlvblock consumer entry
  * @param consumer_tree pointer to tree of consumers
  * @param entries pointer to rfc5444_reader_tlvblock_consumer_entry array
  * @param entrycount number of elements in array
- * @param order order of the consumer
  * @return pointer to rfc5444_reader_tlvblock_consumer,
  *   NULL if an error happened
  */
 static struct rfc5444_reader_tlvblock_consumer *
 _add_consumer(struct rfc5444_reader_tlvblock_consumer *consumer, struct avl_tree *consumer_tree,
-    struct rfc5444_reader_tlvblock_consumer_entry *entries, int entrycount) {
+  struct rfc5444_reader_tlvblock_consumer_entry *entries, int entrycount) {
   struct rfc5444_reader_tlvblock_consumer_entry *e;
   int i, o;
   bool set;
@@ -1281,7 +1297,7 @@ _add_consumer(struct rfc5444_reader_tlvblock_consumer *consumer, struct avl_tree
   list_init_head(&consumer->_consumer_list);
 
   /* generate sorted list of entries */
-  for (i=0; i<entrycount; i++) {
+  for (i = 0; i < entrycount; i++) {
     o = _calc_tlvconsumer_intorder(&entries[i]);
 
     if (i == 0) {
@@ -1314,13 +1330,11 @@ _add_consumer(struct rfc5444_reader_tlvblock_consumer *consumer, struct avl_tree
 
 /**
  * Free a rfc5444_reader_tlvblock_consumer and remove it from its linked list
- * @param parser pointer to parser context
- * @param listhead pointer to listhead pointer
- * @param consumer pointer to rfc5444_reader_tlvblock_consumer
+ * @param consumer_tree tree of tlvblock consumers
+ * @param consumer rfc5444_reader_tlvblock_consumer
  */
 static void
-_free_consumer(struct avl_tree *consumer_tree,
-    struct rfc5444_reader_tlvblock_consumer *consumer) {
+_free_consumer(struct avl_tree *consumer_tree, struct rfc5444_reader_tlvblock_consumer *consumer) {
   /* remove consumer from tree */
   if (avl_is_node_added(&consumer->_node)) {
     avl_remove(consumer_tree, &consumer->_node);
@@ -1345,6 +1359,24 @@ _malloc_tlvblock_entry(void) {
   return calloc(1, sizeof(struct rfc5444_reader_tlvblock_entry));
 }
 
+/**
+ * Free an addressblock entry
+ * @param entry addressblock entry
+ */
+static void
+_free_addrblock_entry(struct rfc5444_reader_addrblock_entry *entry) {
+  free(entry);
+}
+
+/**
+ * Free an tlvblock entry
+ * @param entry tlvblock entry
+ */
+static void
+_free_tlvblock_entry(struct rfc5444_reader_tlvblock_entry *entry) {
+  free(entry);
+}
+
 /**
  * @param v first byte of packet header
  * @return packet header version
